from nose2.tools import such

with such.A("test scenario with errors") as it:

    @it.has_teardown
    def teardown_fail():
        raise RuntimeError("Bad Error in such tearDown!")

    @it.should("check that value == 1")
    def test_passes(case):
        pass


it.createTests(globals())



import unittest


def load_tests(loader, standard_tests, pattern):
    suite = loader.suiteClass()

    class Test(unittest.TestCase):
        def test(self):
            import ltpkg2

            assert ltpkg2.lt(1, 2)

    suite.addTest(Test("test"))
    return suite



from nose2.tests._common import FunctionalTestCase


class TestDunderTestPlugin(FunctionalTestCase):
    def test_dunder(self):
        proc = self.runIn("scenario/dundertest_attribute", "-v")
        self.assertTestRunOutputMatches(proc, stderr="Ran 0 tests")



import logging
import os
import sys
import unittest

from nose2 import events, loader, plugins, runner, session, util

log = logging.getLogger(__name__)
__unittest = True


class PluggableTestProgram(unittest.TestProgram):

    """TestProgram that enables plugins.

    Accepts the same parameters as :class:`unittest.TestProgram`,
    but most of them are ignored as their functions are
    handled by plugins.

    :param module: Module in which to run tests. Default: :func:`__main__`
    :param defaultTest: Default test name. Default: ``None``
    :param argv: Command line args. Default: ``sys.argv``
    :param testRunner: *IGNORED*
    :param testLoader: *IGNORED*
    :param exit: Exit after running tests?
    :param verbosity: Base verbosity
    :param failfast: *IGNORED*
    :param catchbreak: *IGNORED*
    :param buffer: *IGNORED*
    :param plugins: List of additional plugin modules to load
    :param excludePlugins: List of plugin modules to exclude
    :param extraHooks: List of hook names and plugin *instances* to
                       register with the session's hooks system. Each
                       item in the list must be a 2-tuple of
                       (hook name, plugin instance)

    .. attribute :: sessionClass

       The class to instantiate to create a test run configuration
       session. Default: :class:`nose2.session.Session`

    .. attribute :: loaderClass

       The class to instantiate to create a test loader. Default:
       :class:`nose2.loader.PluggableTestLoader`.

       .. warning ::

          Overriding this attribute is the only way to customize
          the test loader class. Passing a test loader to
          :func:`__init__` does not work.

    .. attribute :: runnerClass

       The class to instantiate to create a test runner.  Default:
       :class:`nose2.runner.PluggableTestRunner`.

       .. warning ::

          Overriding this attribute is the only way to customize
          the test runner class. Passing a test runner to
          :func:`__init__` does not work.

    .. attribute :: defaultPlugins

       List of default plugin modules to load.

    """

    sessionClass = session.Session
    _currentSession = None
    loaderClass = loader.PluggableTestLoader
    runnerClass = runner.PluggableTestRunner
    defaultPlugins = plugins.DEFAULT_PLUGINS
    excludePlugins = ()

    # XXX override __init__ to warn that testLoader and testRunner are ignored?
    def __init__(self, **kw):
        plugins = kw.pop("plugins", [])
        exclude = kw.pop("excludePlugins", [])
        hooks = kw.pop("extraHooks", [])
        self.defaultPlugins = list(self.defaultPlugins)
        self.excludePlugins = list(self.excludePlugins)
        self.extraHooks = hooks
        self.defaultPlugins.extend(plugins)
        self.excludePlugins.extend(exclude)
        super().__init__(**kw)

    def parseArgs(self, argv):
        """Parse command line args

        Parses arguments and creates a configuration session,
        then calls :func:`createTests`.

        """
        self.session = self.sessionClass()
        self.__class__._currentSession = self.session

        self.argparse = self.session.argparse  # for convenience

        # XXX force these? or can it be avoided?
        self.testLoader = self.loaderClass(self.session)
        self.session.testLoader = self.testLoader

        # Parse initial arguments like config file paths, verbosity
        self.setInitialArguments()
        # FIXME -h here makes processing stop.
        cfg_args, argv = self.argparse.parse_known_args(argv[1:])
        self.handleCfgArgs(cfg_args)

        # Parse arguments for plugins (if any) and test names
        self.argparse.add_argument("testNames", nargs="*")
        # add help arg now so -h will also print plugin opts
        self.argparse.add_argument(
            "-h", "--help", action="help", help=("Show this help message and exit")
        )
        args, argv = self.argparse.parse_known_args(argv)
        if argv:
            self.argparse.error("Unrecognized arguments: %s" % " ".join(argv))
        self.handleArgs(args)
        self.createTests()

    def setInitialArguments(self):
        """Set pre-plugin command-line arguments.

        This set of arguments is parsed out of the command line
        before plugins are loaded.

        """
        self.argparse.add_argument(
            "-s",
            "--start-dir",
            default=None,
            help="Directory to start discovery ('.' default)",
        )
        self.argparse.add_argument(
            "-t",
            "--top-level-directory",
            "--project-directory",
            help="Top level directory of project (defaults to start dir)",
        )
        self.argparse.add_argument(
            "--config",
            "-c",
            nargs="?",
            action="append",
            default=["unittest.cfg", "nose2.cfg"],
            help="Config files to load, if they exist. ('unittest.cfg' "
            "and 'nose2.cfg' in start directory default)",
        )
        self.argparse.add_argument(
            "--no-user-config",
            action="store_const",
            dest="user_config",
            const=False,
            default=True,
            help="Do not load user config files",
        )
        self.argparse.add_argument(
            "--no-plugins",
            action="store_const",
            dest="load_plugins",
            const=False,
            default=True,
            help="Do not load any plugins. Warning: nose2 does not "
            "do anything if no plugins are loaded",
        )
        self.argparse.add_argument(
            "--plugin",
            action="append",
            dest="plugins",
            default=[],
            help="Load this plugin module.",
        )
        self.argparse.add_argument(
            "--exclude-plugin",
            action="append",
            dest="exclude_plugins",
            default=[],
            help="Do not load this plugin module",
        )
        self.argparse.add_argument(
            "--verbosity",
            type=int,
            help=("Set starting verbosity level (int). " "Applies before -v and -q"),
        )
        self.argparse.add_argument(
            "--verbose",
            "-v",
            action="count",
            default=0,
            help=(
                "Print test case names and statuses. "
                "Use multiple '-v's for higher verbosity."
            ),
        )
        self.argparse.add_argument(
            "--quiet",
            "-q",
            action="count",
            default=0,
            dest="quiet",
            help=("Reduce verbosity. Multiple '-q's result in " "lower verbosity."),
        )
        self.argparse.add_argument(
            "--log-level",
            default=logging.WARN,
            help="Set logging level for message logged to console.",
        )

    def handleCfgArgs(self, cfg_args):
        """Handle initial arguments.

        Handle the initial, pre-plugin arguments parsed out of the
        command line.

        """
        self.session.logLevel = util.parse_log_level(cfg_args.log_level)
        logging.basicConfig(level=self.session.logLevel)
        log.debug("logging initialized %s", cfg_args.log_level)
        if cfg_args.top_level_directory:
            self.session.topLevelDir = cfg_args.top_level_directory
        self.session.loadConfigFiles(*self.findConfigFiles(cfg_args))
        # set verbosity from config + opts
        self.session.setVerbosity(cfg_args.verbosity, cfg_args.verbose, cfg_args.quiet)
        self.session.setStartDir(args_start_dir=cfg_args.start_dir)
        self.session.prepareSysPath()
        if cfg_args.load_plugins:
            self.defaultPlugins.extend(cfg_args.plugins)
            self.excludePlugins.extend(cfg_args.exclude_plugins)
            self.loadPlugins()
        elif cfg_args.plugins or cfg_args.exclude_plugins:
            log.warn(
                "Both '--no-plugins' and '--plugin' or '--exclude-plugin' "
                "specified. No plugins were loaded."
            )

    def findConfigFiles(self, cfg_args):
        """Find available config files"""
        filenames = cfg_args.config[:]
        proj_opts = ("unittest.cfg", "nose2.cfg")
        for fn in proj_opts:
            if cfg_args.top_level_directory:
                fn = os.path.abspath(os.path.join(cfg_args.top_level_directory, fn))
            filenames.append(fn)
        if cfg_args.user_config:
            user_opts = ("~/.unittest.cfg", "~/.nose2.cfg")
            for fn in user_opts:
                filenames.append(os.path.expanduser(fn))
        return filenames

    def handleArgs(self, args):
        """Handle further arguments.

        Handle arguments parsed out of command line after plugins have
        been loaded (and injected their argument configuration).

        """
        self.testNames = args.testNames
        self.session.hooks.handleArgs(events.CommandLineArgsEvent(args=args))

    def loadPlugins(self):
        """Load available plugins


        :func:`self.defaultPlugins`` and :func:`self.excludePlugins` are passed
        to the session to alter the list of plugins that will be
        loaded.

        This method also registers any (hook, plugin) pairs set in
        ``self.hooks``. This is a good way to inject plugins that fall
        outside of the normal loading procedure, for example, plugins
        that need some runtime information that can't easily be
        passed to them through the configuration system.

        """
        self.session.loadPlugins(self.defaultPlugins, self.excludePlugins)
        for method_name, plugin in self.extraHooks:
            self.session.hooks.register(method_name, plugin)

    def createTests(self):
        """Create top-level test suite"""
        event = events.CreateTestsEvent(self.testLoader, self.testNames, self.module)
        result = self.session.hooks.createTests(event)
        if event.handled:
            test = result
        else:
            log.debug("Create tests from %s/%s", self.testNames, self.module)
            test = self.testLoader.loadTestsFromNames(self.testNames, self.module)

        event = events.CreatedTestSuiteEvent(test)
        result = self.session.hooks.createdTestSuite(event)
        if event.handled:
            test = result
        self.test = test

    def runTests(self):
        """Run tests"""
        # fire plugin hook
        runner = self._makeRunner()
        try:
            self.result = runner.run(self.test)
        except Exception as e:
            log.exception("Internal Error")
            sys.stderr.write("Internal Error: runTests aborted: %s\n" % (e))
            if self.exit:
                sys.exit(1)
        if self.exit:
            sys.exit(not self.result.wasSuccessful())

    def _makeRunner(self):
        runner = self.runnerClass(self.session)
        event = events.RunnerCreatedEvent(runner)
        self.session.hooks.runnerCreated(event)
        self.session.testRunner = event.runner
        return event.runner

    @classmethod
    def getCurrentSession(cls):
        """Returns the current session, or ``None`` if no
        :class:`nose2.session.Session` is running.

        """
        return cls._currentSession


main = PluggableTestProgram


def discover(*args, **kwargs):
    """Main entry point for test discovery.

    Running discover calls :class:`nose2.main.PluggableTestProgram`,
    passing through all arguments and keyword arguments **except module**:
    ``module`` is discarded, to force test discovery.

    """
    kwargs["module"] = None
    return main(*args, **kwargs)



from nose2.tests._common import FunctionalTestCase


class TestPluggableTestProgram(FunctionalTestCase):
    def test_run_in_empty_dir_succeeds(self):
        proc = self.runIn("scenario/no_tests")
        stdout, stderr = proc.communicate()
        self.assertEqual(proc.poll(), 0, stderr)

    def test_extra_hooks(self):
        class Check:
            ran = False

            def startTestRun(self, event):
                self.ran = True

        check = Check()
        proc = self.runIn("scenario/no_tests", extraHooks=[("startTestRun", check)])
        stdout, stderr = proc.communicate()
        self.assertEqual(proc.poll(), 0, stderr)
        assert check.ran, "Extra hook did not execute"

    def test_run_in_module_from_its_main(self):
        proc = self.runModuleAsMain("scenario/one_test/tests.py")
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertEqual(proc.poll(), 0)



import re

from nose2.tests._common import FunctionalTestCase


class TestPrintHooksPlugin(FunctionalTestCase):
    def test_invocation_by_double_dash_option(self):
        proc = self.runIn(
            "scenario/no_tests", "--plugin=nose2.plugins.printhooks", "--print-hooks"
        )
        match = re.compile(
            "\n" + r"handleArgs: CommandLineArgsEvent\(handled=False, args="
        )
        self.assertTestRunOutputMatches(proc, stderr=match)
        self.assertEqual(proc.poll(), 0)

    def test_invocation_by_single_dash_option(self):
        proc = self.runIn(
            "scenario/no_tests", "--plugin=nose2.plugins.printhooks", "-P"
        )
        match = re.compile(
            "\n" + r"handleArgs: CommandLineArgsEvent\(handled=False, args="
        )
        self.assertTestRunOutputMatches(proc, stderr=match)
        self.assertEqual(proc.poll(), 0)

    def test_nested_hooks_are_indented(self):
        proc = self.runIn(
            "scenario/no_tests", "--plugin=nose2.plugins.printhooks", "--print-hooks"
        )
        match = re.compile("\n  handleFile: ")
        self.assertTestRunOutputMatches(proc, stderr=match)
        self.assertEqual(proc.poll(), 0)



import unittest

from nose2 import events, exceptions, loader, session
from nose2.plugins import layers
from nose2.tests._common import TestCase


class TestLayers(TestCase):
    tags = ["unit"]

    def setUp(self):
        self.session = session.Session()
        self.loader = loader.PluggableTestLoader(session=self.session)
        self.session.testLoader = self.loader
        self.plugin = layers.Layers(session=self.session)

    def test_simple_layer_inheritance(self):
        class L1:
            pass

        class L2(L1):
            pass

        class T1(unittest.TestCase):
            layer = L1

            def test(self):
                pass

        class T2(unittest.TestCase):
            layer = L2

            def test(self):
                pass

        suite = unittest.TestSuite([T2("test"), T1("test")])
        event = events.StartTestRunEvent(None, suite, None, 0, None)
        self.plugin.startTestRun(event)
        expect = [
            [
                "test (nose2.tests.unit.test_layers_plugin.T1)",
                ["test (nose2.tests.unit.test_layers_plugin.T2)"],
            ]
        ]
        self.assertEqual(self.names(event.suite), expect)

    def test_multiple_inheritance(self):
        class L1:
            pass

        class L2(L1):
            pass

        class L3(L1):
            pass

        class T1(unittest.TestCase):
            layer = L1

            def test(self):
                pass

        class T2(unittest.TestCase):
            layer = L2

            def test(self):
                pass

        class T3(unittest.TestCase):
            layer = L3

            def test(self):
                pass

        suite = unittest.TestSuite([T2("test"), T1("test"), T3("test")])
        event = events.StartTestRunEvent(None, suite, None, 0, None)
        self.plugin.startTestRun(event)
        expect = [
            [
                "test (nose2.tests.unit.test_layers_plugin.T1)",
                ["test (nose2.tests.unit.test_layers_plugin.T2)"],
                ["test (nose2.tests.unit.test_layers_plugin.T3)"],
            ]
        ]
        self.assertEqual(self.names(event.suite), expect)

    def test_deep_inheritance(self):
        class L1:
            pass

        class L2(L1):
            pass

        class L3(L1):
            pass

        class L4(L2, L1):
            pass

        class L5(L4):
            pass

        class T1(unittest.TestCase):
            layer = L1

            def test(self):
                pass

        class T2(unittest.TestCase):
            layer = L2

            def test(self):
                pass

        class T3(unittest.TestCase):
            layer = L3

            def test(self):
                pass

        class T4(unittest.TestCase):
            layer = L4

            def test(self):
                pass

        class T5(unittest.TestCase):
            layer = L5

            def test(self):
                pass

        suite = unittest.TestSuite(
            [T2("test"), T1("test"), T3("test"), T4("test"), T5("test")]
        )
        event = events.StartTestRunEvent(None, suite, None, 0, None)
        self.plugin.startTestRun(event)
        expect = [
            [
                "test (nose2.tests.unit.test_layers_plugin.T1)",
                [
                    "test (nose2.tests.unit.test_layers_plugin.T2)",
                    [
                        "test (nose2.tests.unit.test_layers_plugin.T4)",
                        ["test (nose2.tests.unit.test_layers_plugin.T5)"],
                    ],
                ],
                ["test (nose2.tests.unit.test_layers_plugin.T3)"],
            ]
        ]
        self.assertEqual(self.names(event.suite), expect)

    def test_mixed_layers_no_layers(self):
        class L1:
            pass

        class L2(L1):
            pass

        class T1(unittest.TestCase):
            layer = L1

            def test(self):
                pass

        class T2(unittest.TestCase):
            layer = L2

            def test(self):
                pass

        class T3(unittest.TestCase):
            def test(self):
                pass

        suite = unittest.TestSuite([T2("test"), T1("test"), T3("test")])
        event = events.StartTestRunEvent(None, suite, None, 0, None)
        self.plugin.startTestRun(event)
        expect = [
            "test (nose2.tests.unit.test_layers_plugin.T3)",
            [
                "test (nose2.tests.unit.test_layers_plugin.T1)",
                ["test (nose2.tests.unit.test_layers_plugin.T2)"],
            ],
        ]
        self.assertEqual(self.names(event.suite), expect)

    def test_ordered_layers(self):
        class L1:
            pass

        class L2(L1):
            position = 1

        class L3(L1):
            position = 2

        class L4(L1):
            position = 3

        class L5(L2):
            position = 4

        class T1(unittest.TestCase):
            layer = L1

            def test(self):
                pass

        class T2(unittest.TestCase):
            layer = L2

            def test(self):
                pass

        class T3(unittest.TestCase):
            layer = L3

            def test(self):
                pass

        class T4(unittest.TestCase):
            layer = L4

            def test(self):
                pass

        class T5(unittest.TestCase):
            layer = L5

            def test(self):
                pass

        suite = unittest.TestSuite(
            [T2("test"), T1("test"), T3("test"), T4("test"), T5("test")]
        )
        event = events.StartTestRunEvent(None, suite, None, 0, None)
        self.plugin.startTestRun(event)
        expect = [
            [
                "test (nose2.tests.unit.test_layers_plugin.T1)",
                [
                    "test (nose2.tests.unit.test_layers_plugin.T2)",
                    ["test (nose2.tests.unit.test_layers_plugin.T5)"],
                ],
                ["test (nose2.tests.unit.test_layers_plugin.T3)"],
                ["test (nose2.tests.unit.test_layers_plugin.T4)"],
            ]
        ]
        self.assertEqual(self.names(event.suite), expect)

    def test_mixin_in_top_layer(self):
        class M1:
            pass

        class L1:
            mixins = (M1,)

        class T1(unittest.TestCase):
            layer = L1

            def test(self):
                pass

        suite = unittest.TestSuite([T1("test")])
        event = events.StartTestRunEvent(None, suite, None, 0, None)
        self.plugin.startTestRun(event)
        expect = [[["test (nose2.tests.unit.test_layers_plugin.T1)"]]]  # M1  # L1  # T1
        self.assertEqual(self.names(event.suite), expect)

    def test_mixin_in_inner_layer(self):
        class M1:
            pass

        class L1:
            pass

        class L2(L1):
            mixins = (M1,)

        class T1(unittest.TestCase):
            layer = L1

            def test(self):
                pass

        class T2(unittest.TestCase):
            layer = L2

            def test(self):
                pass

        suite = unittest.TestSuite([T1("test"), T2("test")])
        event = events.StartTestRunEvent(None, suite, None, 0, None)
        self.plugin.startTestRun(event)
        expect = [  # L1
            [
                "test (nose2.tests.unit.test_layers_plugin.T1)",
                # M1
                [["test (nose2.tests.unit.test_layers_plugin.T2)"]],  # L2
            ]
        ]
        self.assertEqual(self.names(event.suite), expect)

    def test_mixin_inheritance(self):
        # without mixin
        # L1
        #  -> L3
        #   -> L4
        #    -> L5
        # L2
        #  -> L6
        #
        # with mixin new behavior:
        #   the mixin (L4, which comes with L3 and L1)
        #   is inserted after the parent layer (L2).
        # L2
        #  -> L1
        #   -> L3
        #    -> L4
        #     -> L6
        #     -> L5
        #
        # with mixin old behavior
        #   the mixin (L4, which comes with L3 and L1)
        #   is inserted before the parent layer (L2).
        # L1
        #  -> L3
        #   -> L4
        #    -> L2
        #     -> L6
        #    -> L5
        class L1:
            pass

        class L2:  # a mixin, doesn't share a base w/L1
            pass

        class L3(L1):
            pass

        class L4(L3):
            pass

        class L5(L4):
            pass

        class L6(L2):
            mixins = (L4,)

        class T1(unittest.TestCase):
            layer = L1

            def test(self):
                pass

        class T3(unittest.TestCase):
            layer = L3

            def test(self):
                pass

        class T4(unittest.TestCase):
            layer = L4

            def test(self):
                pass

        class T5(unittest.TestCase):
            layer = L5

            def test(self):
                pass

        class T6(unittest.TestCase):
            layer = L6

            def test(self):
                pass

        suite = unittest.TestSuite(
            [T6("test"), T1("test"), T3("test"), T4("test"), T5("test")]
        )
        event = events.StartTestRunEvent(None, suite, None, 0, None)
        self.plugin.startTestRun(event)
        expect = [  # L2
            [  # L1
                [
                    "test (nose2.tests.unit.test_layers_plugin.T1)",  # T1
                    # L3
                    [
                        "test (nose2.tests.unit.test_layers_plugin.T3)",  # T3
                        # L4
                        [
                            "test (nose2.tests.unit.test_layers_plugin.T4)",  # T4
                            # L5
                            ["test (nose2.tests.unit.test_layers_plugin.T5)"],
                            # L6
                            ["test (nose2.tests.unit.test_layers_plugin.T6)"],
                        ],
                    ],
                ]
            ]
        ]
        self.assertEqual(self.names(event.suite), expect)

    def test_invalid_top_layer(self):
        # in python 3, L1 will automatically have `object` has base, so
        # this test does not make sense, and will actually fail.
        return

        class L1:
            pass

        class T1(unittest.TestCase):
            layer = L1

            def test(self):
                pass

        suite = unittest.TestSuite([T1("test")])
        event = events.StartTestRunEvent(None, suite, None, 0, None)
        with self.assertRaises(exceptions.LoadTestsFailure):
            self.plugin.startTestRun(event)

    def names(self, suite):
        return list(self.iternames(suite))

    def iternames(self, suite):
        for t in suite:
            if isinstance(t, unittest.TestCase):
                test_module = t.__class__.__module__
                test_class = t.__class__.__name__
                test_method = t._testMethodName
                yield f"{test_method} ({test_module}.{test_class})"
            else:
                yield list(self.iternames(t))

    def _listset(self, lst):
        n = set()
        for t in lst:
            if isinstance(t, list):
                n.add(self._listset(t))
            else:
                n.add(t)
        return frozenset(n)



import logging
import unittest

import nose2

log = logging.getLogger(__name__)


class Test(unittest.TestCase):
    def test_logging_keeps_copies_of_mutable_objects(self):
        d = {}
        log.debug("foo: %s", d)
        d["bar"] = "baz"
        self.assertTrue(False)


if __name__ == "__main__":
    nose2.main()



import unittest


class Case(unittest.TestCase):
    def test_subtest_success(self):
        for i in range(3):
            with self.subTest(i=i):
                self.assertTrue(i < 3)

    def test_subtest_failure(self):
        for i in range(6):
            with self.subTest(i=i):
                self.assertEqual(i % 2, 0)

    def test_subtest_error(self):
        for i in range(3):
            with self.subTest(i=i):
                raise RuntimeError(i)

    @unittest.expectedFailure
    def test_subtest_expected_failure(self):
        for i in range(6):
            with self.subTest(i=i):
                self.assertEqual(i % 2, 0)

    def test_subtest_message(self):
        for i in range(6):
            with self.subTest("msg", i=i):
                self.assertEqual(i % 2, 0)



"""
This plugin implements :func:`startTestRun`, setting a test executor
(``event.executeTests``) that just collects tests without executing
them. To do so it calls result.startTest, result.addSuccess and
result.stopTest for each test, without calling the test itself.
"""
import unittest

from nose2.events import Plugin

__unittest = True


class CollectOnly(Plugin):

    """Collect but don't run tests"""

    configSection = "collect-only"
    commandLineSwitch = (
        None,
        "collect-only",
        "Collect but do not run tests. With '-v', this will output test names",
    )
    _mpmode = False

    def registerInSubprocess(self, event):
        event.pluginClasses.append(self.__class__)
        self._mpmode = True

    def startTestRun(self, event):
        """Replace ``event.executeTests``"""
        if self._mpmode:
            return
        event.executeTests = self.collectTests

    def startSubprocess(self, event):
        event.executeTests = self.collectTests

    def collectTests(self, suite, result):
        """Collect tests, but don't run them"""
        for test in suite:
            if isinstance(test, unittest.BaseTestSuite):
                self.collectTests(test, result)
                continue
            result.startTest(test)
            result.addSuccess(test)
            result.stopTest(test)



THINGS = []


def setUpModule():
    THINGS.append(1)


def tearDownModule():
    while THINGS:
        THINGS.pop()


def test(p):
    assert THINGS, "setup didn't run I think"


test.paramList = (1,)



import os
import sys
import unittest

from nose2 import events, loader, runner, session
from nose2.main import PluggableTestProgram

__unittest = True


def collector():
    """
    This is the entry point used by setuptools, as in::

      python setup.py test

    """

    class Test(unittest.TestCase):
        def run(self, result_):
            ok = self._collector(result_)
            sys.exit(not ok)

        def _get_objects(self):
            ssn = session.Session()
            ldr = loader.PluggableTestLoader(ssn)
            rnr = runner.PluggableTestRunner(ssn)
            return ssn, ldr, rnr

        def _collector(self, result_):
            ssn, ldr, rnr = self._get_objects()

            ssn.testLoader = ldr
            ssn.loadConfigFiles(
                "unittest.cfg",
                "nose2.cfg",
                "setup.cfg",
                os.path.expanduser("~/.unittest.cfg"),
                os.path.expanduser("~/.nose2.cfg"),
            )
            ssn.setStartDir()
            ssn.prepareSysPath()
            ssn.loadPlugins(PluggableTestProgram.defaultPlugins)

            # TODO: refactor argument parsing to make it possible to feed CLI
            # args to plugins via this path (currently done in
            # PluggableTestProgram)
            # in order to do this, it seems like features in
            # PluggableTestProgram need to be factored out into some source
            # from which both it and this dummy test case can invoke them
            #
            # this is the disabled feature:
            # ssn.hooks.handleArgs(events.CommandLineArgsEvent(...))
            #
            # this means that there may be plugins which don't work under
            # setuptools invocation because they expect to get handleArgs
            # triggered (e.g. older versions of the coverage plugin)

            # FIXME: this is all a great-big DRY violation when compared with
            # PluggableTestProgram

            # create the testsuite, and make sure the createTests event gets
            # triggered, as some plugins expect it
            # just doing `ldr.loadTestsFromNames` works, but leaves some
            # plugins in the lurch
            event = events.CreateTestsEvent(ldr, [], None)
            result = ssn.hooks.createTests(event)
            if event.handled:
                test = event
            else:
                test = ldr.loadTestsFromNames([], None)

            # fire the "createdTestSuite" event for plugins to handle
            # as above, we can get away without this, but some plugins will
            # expect it
            event = events.CreatedTestSuiteEvent(test)
            result = ssn.hooks.createdTestSuite(event)
            if event.handled:
                test = result

            rslt = rnr.run(test)
            return rslt.wasSuccessful()

    return Test("_collector")



import logging
from unittest import TestSuite

from nose2.events import Plugin

log = logging.getLogger(__name__)
undefined = object()


class AttributeSelector(Plugin):

    """Filter tests by attribute"""

    def __init__(self):
        self.attribs = []
        self.eval_attribs = []
        self.addArgument(
            self.attribs, "A", "attribute", "Select tests with matching attribute"
        )
        self.addArgument(
            self.eval_attribs,
            "E",
            "eval-attribute",
            "Select tests for whose attributes the "
            "given Python expression evaluates to ``True``",
        )

    def handleArgs(self, args):
        """Register if any attribs defined"""
        if self.attribs or self.eval_attribs:
            self.register()

    def moduleLoadedSuite(self, event):
        """Filter event.suite by specified attributes"""
        log.debug("Attribute selector attribs %s/%s", self.attribs, self.eval_attribs)
        attribs = []
        for attr in self.eval_attribs:

            def eval_in_context(expr, obj):
                try:
                    return eval(expr, None, ContextHelper(obj))
                except Exception as e:
                    log.warning("%s raised exception %s with test %s", expr, e, obj)
                    return False

            attribs.append([(attr, eval_in_context)])
        for attr in self.attribs:
            # all attributes within an attribute group must match
            attr_group = []
            for attrib in attr.strip().split(","):
                # don't die on trailing comma
                if not attrib:
                    continue
                items = attrib.split("=", 1)
                if len(items) > 1:
                    # "name=value"
                    # -> 'str(obj.name) == value' must be True
                    key, value = items
                else:
                    key = items[0]
                    if key[0] == "!":
                        # "!name"
                        # 'bool(obj.name)' must be False
                        key = key[1:]
                        value = False
                    else:
                        # "name"
                        # -> 'bool(obj.name)' must be True
                        value = True
                attr_group.append((key, value))
            attribs.append(attr_group)
        if not attribs:
            return

        event.suite = self.filterSuite(event.suite, attribs)

    def filterSuite(self, suite, attribs):
        # FIXME probably need to copy or something to allow suites w/custom attrs to
        # work or iter and remove instead of recreating
        new_suite = suite.__class__()

        for test in suite:
            if isinstance(test, TestSuite):
                new_suite.addTest(self.filterSuite(test, attribs))
            elif self.validateAttrib(test, attribs):
                new_suite.addTest(test)
        return new_suite

    def validateAttrib(self, test, attribs):
        any_ = False
        for group in attribs:
            match = True
            for key, value in group:
                neg = False
                if key.startswith("!"):
                    neg, key = True, key[1:]
                obj_value = _get_attr(test, key)
                if callable(value):
                    if not value(key, test):
                        match = False
                        break
                elif value is True:
                    # value must exist and be True
                    if not bool(obj_value):
                        match = False
                        break
                elif value is False:
                    # value must not exist or be False
                    if bool(obj_value):
                        match = False
                        break
                elif type(obj_value) in (list, tuple):
                    # value must be found in the list attribute
                    found = str(value).lower() in [str(x).lower() for x in obj_value]
                    if found and neg:
                        match = False
                        break
                    elif not found and not neg:
                        match = False
                        break
                else:
                    # value must match, convert to string and compare
                    if (
                        value != obj_value
                        and str(value).lower() != str(obj_value).lower()
                    ):
                        match = False
                        break
            any_ = any_ or match
        return any_


# helpers


def _get_attr(test, key):
    # FIXME for vals that are lists (or just mutable?), combine all levels
    val = getattr(test, key, undefined)
    if val is not undefined:
        return val
    if hasattr(test, "_testFunc"):
        val = getattr(test._testFunc, key, undefined)
        if val is not undefined:
            return val
    elif hasattr(test, "_testMethodName"):
        # testclasses support
        if test.__class__.__name__ == "_MethodTestCase":
            meth = getattr(test.obj, test.method, undefined)
        else:
            meth = getattr(test, test._testMethodName, undefined)
        if meth is not undefined:
            val = getattr(meth, key, undefined)
            if val is not undefined:
                return val


class ContextHelper:
    def __init__(self, obj):
        self.obj = obj

    def __getitem__(self, name):
        return _get_attr(self.obj, name)



from nose2.tools import such

with such.A("test scenario with errors") as it:

    @it.has_setup
    def set_value():
        it.value = 1

    @it.should("check that value == 1")
    def test_passes(case):
        case.assertEqual(it.value, 1)

    @it.should("check that value == 2 and fail")
    def test_fails(case):
        case.assertEqual(it.value, 2)

    @it.should("check for an attribute that does not exist and raise an error")
    def test_err(case):
        case.assertEqual(it.mulch, "pine")


it.createTests(globals())



class Test:
    def __init__(self):
        raise RuntimeError(
            "Something bad happened but other tests should still be run!"
        )

    def test(self):
        raise RuntimeError(
            "Something bad happened but other tests should still be run! RUNNING"
        )



"""
Capture log messages during test execution, appending them to the
error reports of failed tests.

This plugin implements :func:`startTestRun`, :func:`startTest`,
:func:`stopTest`, :func:`setTestOutcome`, and :func:`outcomeDetail` to
set up a logging configuration that captures log messages during test
execution, and appends them to error reports for tests that fail or
raise exceptions.

"""
import logging
import threading
from logging.handlers import BufferingHandler

from nose2.events import Plugin
from nose2.util import ln, parse_log_level

log = logging.getLogger(__name__)
__unittest = True


class LogCapture(Plugin):

    """Capture log messages during test execution"""

    configSection = "log-capture"
    commandLineSwitch = (None, "log-capture", "Enable log capture")
    logformat = "%(name)s: %(levelname)s: %(message)s"
    logdatefmt = None
    clear = False
    filters = ["-nose"]

    def __init__(self):
        self.logformat = self.config.as_str("format", self.logformat)
        self.logdatefmt = self.config.as_str("date-format", self.logdatefmt)
        self.filters = self.config.as_list("filter", self.filters)
        self.clear = self.config.as_bool("clear-handlers", self.clear)
        self.loglevel = parse_log_level(self.config.as_str("log-level", "NOTSET"))
        self.handler = MyMemoryHandler(
            1000, self.logformat, self.logdatefmt, self.filters
        )

    def registerInSubprocess(self, event):
        event.pluginClasses.append(self.__class__)

    def startSubprocess(self, event):
        self._setupLoghandler()

    def startTestRun(self, event):
        """Set up logging handler"""
        self._setupLoghandler()

    def startTest(self, event):
        """Set up handler for new test"""
        self._setupLoghandler()

    def setTestOutcome(self, event):
        """Store captured log messages in ``event.metadata``"""
        self._addCapturedLogs(event)

    def stopTest(self, event):
        """Clear captured messages, ready for next test"""
        self.handler.truncate()

    def outcomeDetail(self, event):
        """Append captured log messages to ``event.extraDetail``"""
        logs = event.outcomeEvent.metadata.get("logs", None)
        if logs:
            event.extraDetail.append(ln(">> begin captured logging <<"))
            event.extraDetail.extend(logs)
            event.extraDetail.append(ln(">> end captured logging <<"))

    def _setupLoghandler(self):
        # setup our handler with root logger
        root_logger = logging.getLogger()
        if self.clear:
            if hasattr(root_logger, "handlers"):
                for handler in root_logger.handlers:
                    root_logger.removeHandler(handler)
            for logger in logging.Logger.manager.loggerDict.values():
                if hasattr(logger, "handlers"):
                    for handler in logger.handlers:
                        logger.removeHandler(handler)
        # make sure there isn't one already
        # you can't simply use "if self.handler not in root_logger.handlers"
        # since at least in unit tests this doesn't work --
        # LogCapture() is instantiated for each test case while root_logger
        # is module global
        # so we always add new MyMemoryHandler instance
        for handler in root_logger.handlers[:]:
            if isinstance(handler, MyMemoryHandler):
                root_logger.handlers.remove(handler)
        root_logger.addHandler(self.handler)
        root_logger.setLevel(self.loglevel)

    def _addCapturedLogs(self, event):
        format = self.handler.format
        records = [format(r) for r in self.handler.buffer]
        if "logs" in event.metadata:
            event.metadata["logs"].extend(records)
        else:
            event.metadata["logs"] = records


class FilterSet:
    def __init__(self, filter_components):
        self.inclusive, self.exclusive = self._partition(filter_components)

    @staticmethod
    def _partition(components):
        inclusive, exclusive = [], []
        for component in components:
            if component.startswith("-"):
                exclusive.append(component[1:])
            else:
                inclusive.append(component)
        return inclusive, exclusive

    def allow(self, record):
        """returns whether this record should be printed"""
        if not self:
            # nothing to filter
            return True
        return self._allow(record) and not self._deny(record)

    @staticmethod
    def _any_match(matchers, record):
        """return the bool of whether `record` starts with
        any item in `matchers`"""

        def record_matches_key(key):
            return record == key or record.startswith(key + ".")

        return any(map(record_matches_key, matchers))

    def _allow(self, record):
        if not self.inclusive:
            return True
        return self._any_match(self.inclusive, record)

    def _deny(self, record):
        if not self.exclusive:
            return False
        return self._any_match(self.exclusive, record)


class MyMemoryHandler(BufferingHandler):
    def __init__(self, capacity, logformat, logdatefmt, filters):
        BufferingHandler.__init__(self, capacity)
        fmt = logging.Formatter(logformat, logdatefmt)
        self.setFormatter(fmt)
        self.filterset = FilterSet(filters)

    def flush(self):
        pass  # do nothing

    def truncate(self):
        self.buffer = []

    def filter(self, record):
        return self.filterset.allow(record.name)

    def emit(self, record):
        # take a snapshot of the potentially mutable arguments
        record.msg = record.getMessage()
        record.args = {}
        BufferingHandler.emit(self, record)

    def __getstate__(self):
        state = self.__dict__.copy()
        del state["lock"]
        return state

    def __setstate__(self, state):
        self.__dict__.update(state)
        self.lock = threading.RLock()



from nose2 import util
from nose2.tests._common import TestCase, support_file


class UtilTests(TestCase):
    def test_name_from_path(self):
        test_module = support_file("scenario/tests_in_package/pkg1/test/test_things.py")
        test_package_path = support_file("scenario/tests_in_package")
        self.assertEqual(
            util.name_from_path(test_module),
            ("pkg1.test.test_things", test_package_path),
        )

    def test_non_ascii_output(self):
        class D:
            def __init__(self):
                self.out = []

            def write(self, arg):
                self.out.append(arg)

        stream = D()
        decorated = util._WritelnDecorator(stream)
        string = "\u00dcnic\u00f6de"
        decorated.write(string)
        str("".join(stream.out))



from nose2.tests._common import FunctionalTestCase


class TestAttribPlugin(FunctionalTestCase):
    def test_simple_true(self):
        proc = self.runIn(
            "scenario/tests_in_package",
            "-v",
            "--plugin=nose2.plugins.attrib",
            "-A",
            "a",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 4 tests")
        self.assertTestRunOutputMatches(proc, stderr="test_params_method")
        self.assertTestRunOutputMatches(proc, stderr="test_func")

    def test_simple_false(self):
        proc = self.runIn(
            "scenario/tests_in_package",
            "-v",
            "--plugin=nose2.plugins.attrib",
            "-A",
            "!a",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 21 tests")

    def test_simple_value(self):
        proc = self.runIn(
            "scenario/tests_in_package",
            "-v",
            "--plugin=nose2.plugins.attrib",
            "-A",
            "b=2",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 2 tests")

    def test_list_value(self):
        proc = self.runIn(
            "scenario/tests_in_package",
            "-v",
            "--plugin=nose2.plugins.attrib",
            "-A",
            "tags=func",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 8 tests")
        self.assertTestRunOutputMatches(proc, stderr="test_params_func")
        self.assertTestRunOutputMatches(proc, stderr="test_func")
        self.assertTestRunOutputMatches(proc, stderr="test_gen")

    def test_list_value_negation(self):
        proc = self.runIn(
            "scenario/tests_in_package",
            "-v",
            "--plugin=nose2.plugins.attrib",
            "-A",
            "!tags=func",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 8 tests")
        self.assertTestRunOutputMatches(proc, stderr="test_gen_method")
        self.assertTestRunOutputMatches(proc, stderr="test_params_method")
        self.assertTestRunOutputMatches(proc, stderr="test_ok")
        self.assertTestRunOutputMatches(proc, stderr="test_failed")
        self.assertTestRunOutputMatches(proc, stderr="test_skippy")
        self.assertTestRunOutputMatches(proc, stderr="test_typeerr")

    def test_eval_expr(self):
        proc = self.runIn(
            "scenario/tests_in_package",
            "-v",
            "--plugin=nose2.plugins.attrib",
            "-E",
            "a == b and a",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertTestRunOutputMatches(proc, stderr="skippy")



# Adapted from unittest2/events.py from the unittest2 plugins branch.
# This module contains some code copied from unittest2/events.py and other
# code developed in reference to that module and others within unittest2.
# unittest2 is Copyright (c) 2001-2010 Python Software Foundation; All
# Rights Reserved. See: http://docs.python.org/license.html

import argparse
import logging
import unittest

from nose2 import config, util

log = logging.getLogger(__name__)
__unittest = True

# FIXME decide on a real rule for camelCase vs under_score and stick with it.

# XXX I'd rather move this stuff to Plugin.__init__ and
# have __init__ call self.configure() or something after the
# initial setup, but that would further break compatibility
# with the unittest2 plugins branch Plugin class.


class PluginMeta(type):
    def __call__(cls, *args, **kwargs):
        session = kwargs.pop("session", None)
        instance = object.__new__(cls, *args, **kwargs)
        instance.session = session
        instance.config = config.Config([])

        config_section = getattr(instance, "configSection", None)
        switch = getattr(instance, "commandLineSwitch", None)

        if session is not None and config_section is not None:
            instance.config = session.get(config_section)

        always_on = instance.config.as_bool("always-on", default=instance.alwaysOn)

        instance.__init__(*args, **kwargs)

        if always_on:
            instance.register()

        if switch is not None:
            short_opt, long_opt, help = switch
            if always_on:  # always-on plugins should hide their options
                help = argparse.SUPPRESS
            instance.addOption(instance._register_cb, short_opt, long_opt, help)

        return instance


class Plugin(metaclass=PluginMeta):

    """Base class for nose2 plugins

    All nose2 plugins must subclass this class.

    .. attribute :: session

       The :class:`nose2.session.Session` under which the plugin
       has been loaded.

    .. attribute :: config

       The :class:`nose2.config.Config` representing the plugin's
       config section as loaded from the session's config files.

    .. attribute :: commandLineSwitch

       A tuple of (short opt, long opt, help text) that defines a command
       line flag that activates this plugin. The short opt may be ``None``. If
       defined, it must be a single upper-case character. Both short and
       long opt must *not* start with dashes.

       Example::

         commandLineSwitch = ('B', 'buffer-output', 'Buffer output during
         tests')

    .. attribute :: configSection

       The name config file section to load into this plugin's config.

    .. attribute :: alwaysOn

       If this plugin should automatically register itself, set alwaysOn to
       ``True``. Default is ``False``.

    .. note ::

       Plugins that use config values from config files and want to
       use the nose2 sphinx extension to automatically generate
       documentation *must* extract all config values from
       ``self.config`` in ``__init__``. Otherwise the extension will
       not be able to detect the config keys that the plugin uses.

    """

    alwaysOn = False
    registered = False

    def register(self):
        """Register with appropriate hooks.

        This activates the plugin and enables it to receive events.

        """
        if self.session is None:
            log.warning("Unable to register %s, no session", self)
            return
        self.session.registerPlugin(self)
        self.registered = True

    def addMethods(self, *methods):
        """Add new plugin methods to hooks registry

        Any plugins that are already registered and implement
        a method added here will be registered for that
        method as well.

        """
        for method in methods:
            self.session.hooks.addMethod(method)
        for plugin in self.session.plugins:
            for method in methods:
                if plugin.registered and hasattr(plugin, method):
                    self.session.hooks.register(method, plugin)

    def _register_cb(self, *_):
        if not self.registered:
            self.register()

    def addFlag(self, callback, short_opt, long_opt, help_text=None):
        """Add command-line flag that takes no arguments

        :param callback: Callback function to run when flag is seen.
                         The callback will receive one empty argument.
        :param short_opt: Short option. Must be uppercase, no dashes.
        :param long_opt: Long option. Must not start with dashes
        :param help_text: Help text for users so they know what this
                          flag does.

        """
        self.addOption(callback, short_opt, long_opt, help_text, nargs=0)

    def addArgument(self, callback, short_opt, long_opt, help_text=None):
        """Add command-line option that takes one argument.

        :param callback: Callback function to run when flag is seen.
                         The callback will receive one argument.
        :param short_opt: Short option. Must be uppercase, no dashes.
        :param long_opt: Long option. Must not start with dashes
        :param help_text: Help text for users so they know what this
                          flag does.

        """
        self.addOption(callback, short_opt, long_opt, help_text, nargs=1)

    def addOption(self, callback, short_opt, long_opt, help_text=None, nargs=0):
        """Add command-line option.

        :param callback: Callback function to run when flag is seen.
                         The callback will receive one argument.
                         The "callback" may also be a list, in which
                         case values submitted on the command line
                         will be appended to the list.
        :param short_opt: Short option. Must be uppercase, no dashes.
        :param long_opt: Long option. Must not start with dashes
        :param help_text: Help text for users so they know what this
                          flag does.
        :param nargs: Number of arguments to consume from command line.

        """
        if self.session is None:
            log.warning(
                "Unable to add option %s/%s for %s, no session",
                short_opt,
                long_opt,
                self,
            )
            return

        class CB(argparse.Action):
            def __call__(self, parser, namespace, values, option_string=None):
                if callable(callback):
                    callback(values)
                elif isinstance(callback, list):
                    callback.extend(values)
                else:
                    raise ValueError(
                        "Invalid callback %s for plugin option %s",
                        callback,
                        option_string,
                    )

        opts = []
        if short_opt:
            if short_opt.lower() == short_opt:
                raise ValueError("Lowercase short options are reserved: %s" % short_opt)
            opts.append("-" + short_opt)
        if long_opt:
            opts.append("--" + long_opt)
        self.session.pluginargs.add_argument(
            *opts, action=CB, help=help_text, const=True, nargs=nargs
        )


class Hook:

    """A plugin hook

    Each plugin method in the :class:`nose2.events.PluginInterface` is
    represented at runtime by a Hook instance that lists the plugins
    that should be called by that hook.

    .. attribute :: method

       The name of the method that this Hook represents.

    .. attribute :: plugins

       The list of plugin instances bound to this hook.

    """

    def __init__(self, method):
        self.method = method
        self.plugins = []

    def __call__(self, event):
        for plugin in self.plugins[:]:
            result = getattr(plugin, self.method)(event)
            if event.handled:
                return result

    def append(self, plugin):
        if plugin not in self.plugins:
            self.plugins.append(plugin)


class PluginInterface:

    """Definition of plugin interface.

    Instances of this class contain the methods that may be called,
    and a dictionary of :class:`nose2.events.Hook` instances bound
    to each method.

    In a plugin, PluginInterface instance is typically available as
    self.session.hooks, and plugin hooks may be called on it
    directly::

      event = events.LoadFromModuleEvent(module=the_module)
      self.session.hooks.loadTestsFromModule(event)

    .. attribute :: preRegistrationMethods

       Tuple of methods that are called before registration.

    .. attribute :: methods

       Tuple of available plugin hook methods.

    .. attribute :: hookClass

       Class to instantiate for each hook. Default: :class:`nose2.events.Hook`.

    """

    preRegistrationMethods = ("pluginsLoaded", "handleArgs")
    methods = (
        "loadTestsFromModule",
        "loadTestsFromNames",
        "handleFile",
        "startLayerSetup",
        "startLayerSetupTest",
        "stopLayerSetupTest",
        "stopLayerSetup",
        "startTestRun",
        "startTest",
        "stopTest",
        "startLayerTeardown",
        "startLayerTeardownTest",
        "stopLayerTeardownTest",
        "stopLayerTeardown",
        "loadTestsFromName",
        "loadTestsFromTestCase",
        "stopTestRun",
        "matchPath",
        "matchDirPath",
        "getTestCaseNames",
        "runnerCreated",
        "resultCreated",
        "testOutcome",
        "wasSuccessful",
        "resultStop",
        "setTestOutcome",
        "describeTest",
        "reportStartTest",
        "reportError",
        "reportFailure",
        "reportSkip",
        "reportSuccess",
        "reportExpectedFailure",
        "reportUnexpectedSuccess",
        "reportOtherOutcome",
        "outcomeDetail",
        "beforeErrorList",
        "beforeSummaryReport",
        "afterSummaryReport",
        "beforeInteraction",
        "afterInteraction",
        "createTests",
        "createdTestSuite",
        "afterTestRun",
        "moduleLoadedSuite",
        "handleDir",
        # ... etc?
    )
    hookClass = Hook

    def __init__(self):
        self.hooks = {}

    def addMethod(self, method):
        """Add a method to the available method.

        This allows plugins to register for this method.

        :param method: A method name

        """
        self.methods = self.methods + (method,)

    def register(self, method, plugin):
        """Register a plugin for a method.

        :param method: A method name
        :param plugin: A plugin instance

        """
        self.hooks.setdefault(method, self.hookClass(method)).append(plugin)

    def __getattr__(self, attr):
        return self.hooks.setdefault(attr, self.hookClass(attr))


class Event:

    """Base class for all events.

    .. attribute :: metadata

       Storage for arbitrary information attached to an event.

    .. attribute :: handled

       Set to ``True`` to indicate that a plugin has handled the event,
       and no other plugins or core systems should process it further.

    .. attribute :: version

       Version of the event API. This will be incremented with each
       release of nose2 that changes the API.

    """

    _attrs = ("handled",)
    version = "0.4"

    def __init__(self, **metadata):
        self.handled = False
        self.metadata = {}
        self.metadata.update(metadata)

    def __str__(self):
        return f"{self.__class__.__name__}({self._format()})"

    def __repr__(self):
        return str(self)

    def _format(self):
        return ", ".join([f"{k}={getattr(self, k, None)!r}" for k in self._attrs])

    def __getstate__(self):
        state = self.__dict__.copy()
        # FIXME fails for loadTestsFailure
        if "test" in state:
            test = state["test"]
            state["test"] = util.test_name(test)
            # subtest support
            if isinstance(test, unittest.case._SubTest):
                state["metadata"]["subtest"] = (test._message, test.params)
        if "executeTests" in state:
            state["executeTests"] = None
        if "exc_info" in state and state["exc_info"] is not None:
            ec, ev, tb = state["exc_info"]
            state["exc_info"] = (ec, ev, util.format_traceback(None, (ec, ev, tb)))
        clear = ("loader", "result", "runner")
        for attr in clear:
            if attr in state:
                state[attr] = None
        return state


class PluginsLoadedEvent(Event):

    """Event fired after all plugin classes are loaded.

    .. attribute :: pluginsLoaded

       List of all loaded plugin classes

    """

    _attrs = Event._attrs + ("pluginsLoaded",)

    def __init__(self, pluginsLoaded, **kw):
        self.pluginsLoaded = pluginsLoaded
        super().__init__(**kw)


class RunnerCreatedEvent(Event):

    """Event fired when test runner is created.

    .. attribute :: runner

       Test runner instance. Plugins may replace the test runner by
       setting this attribute to a new test runner instance.

    """

    _attrs = Event._attrs + ("runner",)

    def __init__(self, runner, **kw):
        self.runner = runner
        super().__init__(**kw)


class ResultCreatedEvent(Event):

    """Event fired when test result handler is created.

    .. attribute :: result

       Test result handler instance. Plugins may replace the test
       result by setting this attribute to a new test result instance.

    """

    _attrs = Event._attrs + ("result",)

    def __init__(self, result, **kw):
        self.result = result
        super().__init__(**kw)


class StartLayerSetupEvent(Event):

    """Event fired before running a layer setup.

    .. attribute :: layer

       The current layer instance, for which setup is about to run.
    """

    _attrs = Event._attrs + ("layer",)

    def __init__(self, layer, **kw):
        self.layer = layer
        super().__init__(**kw)


class StopLayerSetupEvent(Event):

    """Event fired after running a layer setup.

    .. attribute :: layer

       The current layer instance, for which setup just ran.
    """

    _attrs = Event._attrs + ("layer",)

    def __init__(self, layer, **kw):
        self.layer = layer
        super().__init__(**kw)


class StartLayerSetupTestEvent(Event):

    """Event fired before test cases setups in layers.

    .. attribute :: layer

       The current layer instance.

    .. attribute :: test

       The test instance for which the setup is about to run.
    """

    _attrs = Event._attrs + ("layer", "test")

    def __init__(self, layer, test, **kw):
        self.layer = layer
        self.test = test
        super().__init__(**kw)


class StopLayerSetupTestEvent(Event):

    """Event fired after test cases setups in layers.

    .. attribute :: layer

       The current layer instance.

    .. attribute :: test

       The test instance for which the setup just finished.
    """

    _attrs = Event._attrs + ("layer", "test")

    def __init__(self, layer, test, **kw):
        self.layer = layer
        self.test = test
        super().__init__(**kw)


class StartLayerTeardownEvent(Event):

    """Event fired before running a layer teardown.

    .. attribute :: layer

       The current layer instance, for which teardown is about to run.
    """

    _attrs = Event._attrs + ("layer",)

    def __init__(self, layer, **kw):
        self.layer = layer
        super().__init__(**kw)


class StopLayerTeardownEvent(Event):

    """Event fired after running a layer teardown.

    .. attribute :: layer

       The current layer instance, for which teardown just ran.
    """

    _attrs = Event._attrs + ("layer",)

    def __init__(self, layer, **kw):
        self.layer = layer
        super().__init__(**kw)


class StartLayerTeardownTestEvent(Event):

    """Event fired before test cases teardowns in layers.

    .. attribute :: layer

       The current layer instance.

    .. attribute :: test

       The test instance for which teardown is about to run.
    """

    _attrs = Event._attrs + ("layer", "test")

    def __init__(self, layer, test, **kw):
        self.layer = layer
        self.test = test
        super().__init__(**kw)


class StopLayerTeardownTestEvent(Event):

    """Event fired after test cases teardowns in layers.

    .. attribute :: layer

       The current layer instance.

    .. attribute :: test

       The test instance for which teardown just ran.
    """

    _attrs = Event._attrs + ("layer", "test")

    def __init__(self, layer, test, **kw):
        self.layer = layer
        self.test = test
        super().__init__(**kw)


class StartTestRunEvent(Event):

    """Event fired when test run is about to start.

    Test collection is complete before this event fires, but
    no tests have yet been executed.

    .. attribute :: runner

       Test runner

    .. attribute :: suite

       Top-level test suite to execute. Plugins can filter this suite,
       or set event.suite to change which tests execute (or how they
       execute).

    .. attribute :: result

       Test result

    .. attribute :: startTime

       Timestamp of test run start

    .. attribute :: executeTests

       Callable that will be used to execute tests.  Plugins may set
       this attribute to wrap or otherwise change test execution. The
       callable must match the signature::

         def execute(suite, result):
             ...

    To prevent normal test execution, plugins may set ``handled`` on
    this event to ``True``. When ``handled`` is true, the test executor
    does not run at all.

    """

    _attrs = Event._attrs + ("runner", "suite", "result", "startTime", "executeTests")

    def __init__(self, runner, suite, result, startTime, executeTests, **kw):
        self.suite = suite
        self.runner = runner
        self.result = result
        self.startTime = startTime
        self.executeTests = executeTests
        super().__init__(**kw)


class StopTestRunEvent(Event):

    """Event fired when test run has stopped.

    .. attribute :: runner

       Test runner

    .. attribute :: result

       Test result

    .. attribute :: stopTime

       Timestamp of test run stop

    .. attribute :: timeTaken

       Number of seconds test run took to execute

    """

    _attrs = Event._attrs + ("runner", "result", "stopTime", "timeTaken")

    def __init__(self, runner, result, stopTime, timeTaken, **kw):
        self.runner = runner
        self.result = result
        self.stopTime = stopTime
        self.timeTaken = timeTaken
        super().__init__(**kw)


class StartTestEvent(Event):

    """Event fired before a test is executed.

    .. attribute :: test

       The test case

    .. attribute :: result

       Test result

    .. attribute :: startTime

       Timestamp of test start

    """

    _attrs = Event._attrs + ("test", "result", "startTime")

    def __init__(self, test, result, startTime, **kw):
        self.test = test
        self.result = result
        self.startTime = startTime
        super().__init__(**kw)


class StopTestEvent(Event):

    """Event fired after a test is executed.

    .. attribute :: test

       The test case

    .. attribute :: result

       Test result

    .. attribute :: stopTime

       Timestamp of test stop

    """

    _attrs = Event._attrs + ("test", "result", "stopTime")

    def __init__(self, test, result, stopTime, **kw):
        self.test = test
        self.result = result
        self.stopTime = stopTime
        super().__init__(**kw)


class TestOutcomeEvent(Event):

    """Event fired when a test completes.

    .. attribute :: test

       The test case

    .. attribute :: result

       Test result

    .. attribute :: outcome

       Description of test outcome. Typically will be one of 'error',
       'failed', 'skipped', 'passed', or 'subtest'.

    .. attribute :: exc_info

       If the test resulted in an exception, the tuple of (exception
       class, exception value, traceback) as returned by
       ``sys.exc_info()``. If the test did not result in an exception,
       ``None``.

    .. attribute :: reason

       For test outcomes that include a reason (``Skips``, for example),
       the reason.

    .. attribute :: expected

       Boolean indicating whether the test outcome was expected. In
       general, all tests are expected to pass, and any other outcome
       will have expected as ``False``. The exceptions to that rule are
       unexpected successes and expected failures.

    .. attribute :: shortLabel

       A short label describing the test outcome. (For example, 'E'
       for errors).

    .. attribute :: longLabel

       A long label describing the test outcome (for example, 'ERROR'
       for errors).

    Plugins may influence how the rest of the system sees the test
    outcome by setting ``outcome`` or ``exc_info`` or ``expected``. They
    may influence how the test outcome is reported to the user by
    setting ``shortLabel`` or ``longLabel``.

    """

    _attrs = Event._attrs + (
        "test",
        "result",
        "outcome",
        "exc_info",
        "reason",
        "expected",
        "shortLabel",
        "longLabel",
    )

    def __init__(
        self,
        test,
        result,
        outcome,
        exc_info=None,
        reason=None,
        expected=False,
        shortLabel=None,
        longLabel=None,
        **kw,
    ):
        self.test = test
        self.result = result
        self.outcome = outcome
        self.exc_info = exc_info
        self.reason = reason
        self.expected = expected
        self.shortLabel = shortLabel
        self.longLabel = longLabel
        super().__init__(**kw)


class LoadFromModuleEvent(Event):

    """Event fired when a test module is loaded.

    .. attribute :: loader

       Test loader instance

    .. attribute :: module

       The module whose tests are to be loaded

    .. attribute :: extraTests

       A list of extra tests loaded from the module. To load tests
       from a module without interfering with other plugins' loading
       activities, append tests to extraTests.

    Plugins may set ``handled`` on this event and return a test suite
    to prevent other plugins from loading tests from the module. If
    any plugin sets ``handled`` to ``True``, ``extraTests`` will be
    ignored.

    """

    _attrs = Event._attrs + ("loader", "module", "extraTests")

    def __init__(self, loader, module, **kw):
        self.loader = loader
        self.module = module
        self.extraTests = []
        super().__init__(**kw)


class ModuleSuiteEvent(Event):
    _attrs = Event._attrs + ("loader", "module", "suite")

    def __init__(self, loader, module, suite, **kw):
        self.loader = loader
        self.module = module
        self.suite = suite
        super().__init__(**kw)


class LoadFromTestCaseEvent(Event):

    """Event fired when tests are loaded from a test case.

    .. attribute :: loader

       Test loader instance

    .. attribute :: testCase

       The :class:`unittest.TestCase` instance being loaded.

    .. attribute :: extraTests

        A list of extra tests loaded from the module. To load tests
        from a test case without interfering with other plugins'
        loading activities, append tests to extraTests.

    Plugins may set ``handled`` on this event and return a test suite
    to prevent other plugins from loading tests from the test case. If
    any plugin sets ``handled`` to ``True``, ``extraTests`` will be
    ignored.

    """

    _attrs = Event._attrs + ("loader", "testCase", "extraTests")

    def __init__(self, loader, testCase, **kw):
        self.loader = loader
        self.testCase = testCase
        self.extraTests = []
        super().__init__(**kw)


class LoadFromNamesEvent(Event):

    """Event fired to load tests from test names.

    .. attribute :: loader

       Test loader instance

    .. attribute :: names

       List of test names. May be empty or ``None``.

    .. attribute :: module

       Module to load from. May be ``None``. If not ``None``, names should be
       considered relative to this module.

    .. attribute :: extraTests

        A list of extra tests loaded from the tests named. To load
        tests from test names without interfering with other plugins'
        loading activities, append tests to extraTests.

    Plugins may set ``handled`` on this event and return a test suite
    to prevent other plugins from loading tests from the test names. If
    any plugin sets ``handled`` to ``True``, ``extraTests`` will be
    ignored.

    """

    _attrs = Event._attrs + ("loader", "names", "module", "extraTests")

    def __init__(self, loader, names, module, **kw):
        self.loader = loader
        self.names = names
        self.module = module
        self.extraTests = []
        super().__init__(**kw)

    def __str__(self):
        return f"LoadFromNames(names={self.names!r}, module={self.module!r})"


class LoadFromNameEvent(Event):

    """Event fired to load tests from test names.

    .. attribute :: loader

       Test loader instance

    .. attribute :: name

       Test name to load

    .. attribute :: module

       Module to load from. May be ``None``. If not ``None``, names should be
       considered relative to this module.

    .. attribute :: extraTests

        A list of extra tests loaded from the name. To load tests
        from a test name without interfering with other plugins'
        loading activities, append tests to extraTests.

    Plugins may set ``handled`` on this event and return a test suite
    to prevent other plugins from loading tests from the test name. If
    any plugin sets ``handled`` to ``True``, ``extraTests`` will be
    ignored.

    """

    _attrs = Event._attrs + ("loader", "name", "module", "extraTests")

    def __init__(self, loader, name, module, **kw):
        self.loader = loader
        self.name = name
        self.module = module
        self.extraTests = []
        super().__init__(**kw)


class HandleFileEvent(Event):

    """Event fired when a non-test file is examined.

    .. note ::

       This event is fired for all processed python files and modules
       including but not limited to the ones that
       match the test file pattern.

    .. attribute :: loader

       Test loader instance

    .. attribute :: name

       File basename

    .. attribute :: path

       Full path to file

    .. attribute :: pattern

       Current test file match pattern

    .. attribute :: topLevelDirectory

       Top-level directory of the test run

    .. attribute :: extraTests

        A list of extra tests loaded from the file. To load tests
        from a file without interfering with other plugins'
        loading activities, append tests to extraTests.

    Plugins may set ``handled`` on this event and return a test suite
    to prevent other plugins from loading tests from the file. If
    any plugin sets ``handled`` to ``True``, ``extraTests`` will be
    ignored.

    """

    _attrs = Event._attrs + ("loader", "name", "path", "pattern", "topLevelDirectory")

    def __init__(self, loader, name, path, pattern, topLevelDirectory, **kw):
        self.extraTests = []
        self.path = path
        self.loader = loader
        self.name = name
        # note: pattern may be None if not called during test discovery
        self.pattern = pattern
        self.topLevelDirectory = topLevelDirectory
        super().__init__(**kw)


class MatchPathEvent(Event):

    """Event fired during file matching.

    Plugins may return ``False`` and set ``handled`` on this event to prevent
    a file from being matched as a test file, regardless of other system
    settings.

    .. attribute :: path

       Full path to the file

    .. attribute :: name

       File basename

    .. attribute :: pattern

       Current test file match pattern

    """

    _attrs = Event._attrs + ("name", "path", "pattern")

    def __init__(self, name, path, pattern, **kw):
        self.path = path
        self.name = name
        self.pattern = pattern
        super().__init__(**kw)


class GetTestCaseNamesEvent(Event):

    """Event fired to find test case names in a test case.

    Plugins may return a list of names and set ``handled`` on this
    event to force test case name selection.


    .. attribute :: loader

       Test loader instance

    .. attribute :: testCase

       The :class:`unittest.TestCase` instance being loaded.

    .. attribute :: testMethodPrefix

       Set this to change the test method prefix. Unless set by a plugin,
       it is ``None``.

    .. attribute :: extraNames

       A list of extra test names to load from the test case. To cause
       extra tests to be loaded from the test case, append the names
       to this list. Note that the names here must be attributes of
       the test case.

    .. attribute :: excludedNames

       A list of names to exclude from test loading. Add names to this
       list to prevent other plugins from loading the named tests.

    .. attribute :: isTestMethod

       Callable that plugins can use to examine test case attributes to
       determine whether nose2 thinks they are test methods.

    """

    _attrs = Event._attrs + (
        "loader",
        "testCase",
        "testMethodPrefix",
        "extraNames",
        "excludedNames",
        "isTestMethod",
    )

    def __init__(self, loader, testCase, isTestMethod, **kw):
        self.loader = loader
        self.testCase = testCase
        self.testMethodPrefix = None
        self.extraNames = []
        self.excludedNames = []
        self.isTestMethod = isTestMethod
        super().__init__(**kw)


class ResultSuccessEvent(Event):

    """Event fired at end of test run to determine success.

    This event fires at the end of the test run and allows
    plugins to determine whether the test run was successful.

    .. attribute :: result

       Test result

    .. attribute :: success

       Set this to ``True`` to indicate that the test run was
       successful. If no plugin sets the ``success`` to
       ``True``, the test run fails.
       Should be initialized to ``None`` to indicate that the status has not
       been set yet (so that plugins can always differentiate an explicit
       failure in an earlier hook from no pass/fail status having been set yet.

    """

    _attrs = Event._attrs + ("result", "success")

    def __init__(self, result, success, **kw):
        self.result = result
        self.success = success
        super().__init__(**kw)


class ResultStopEvent(Event):

    """Event fired when a test run is told to stop.

    Plugins can use this event to prevent other plugins from stopping
    a test run.

    .. attribute :: result

       Test result

    .. attribute :: shouldStop

       Set to ``True`` to indicate that the test run should stop.

    """

    _attrs = Event._attrs + ("result", "shouldStop")

    def __init__(self, result, shouldStop, **kw):
        self.result = result
        self.shouldStop = shouldStop
        super().__init__(**kw)


class DescribeTestEvent(Event):

    """Event fired to get test description.


    .. attribute :: test

       The test case

    .. attribute :: description

       Description of the test case. Plugins can set this to change
       how tests are described in output to users.

    .. attribute :: errorList

       Is the event fired as part of error list output?

    """

    _attrs = Event._attrs + ("test", "description")

    def __init__(self, test, description=None, errorList=False, **kw):
        self.test = test
        self.description = description
        self.errorList = errorList
        super().__init__(**kw)


class OutcomeDetailEvent(Event):

    """Event fired to acquire additional details about test outcome.

    .. attribute :: outcomeEvent

       A :class:`nose2.events.TestOutcomeEvent` instance holding
       the test outcome to be described.

    .. attribute :: extraDetail

       Extra detail lines to be appended to test outcome
       output. Plugins can append lines (of strings) to this list to
       include their extra information in the error list report.

    """

    _attrs = Event._attrs + ("outcomeEvent", "extraDetail")

    def __init__(self, outcomeEvent, **kw):
        self.outcomeEvent = outcomeEvent
        self.extraDetail = []
        super().__init__(**kw)


class ReportSummaryEvent(Event):

    """Event fired before and after summary report.

    .. attribute :: stopTestEvent

       A :class:`nose2.events.StopTestEvent` instance.

    .. attribute :: stream

       The output stream. Plugins can set this to change or capture
       output.

    .. attribute :: reportCategories

       Dictionary of report category and test events captured in that
       category. Default categories include 'errors', 'failures',
       'skipped', 'expectedFails', and 'unexpectedSuccesses'. Plugins
       may add their own categories.

    """

    _attrs = Event._attrs + ("stopTestEvent", "stream", "reportCategories")

    def __init__(self, stopTestEvent, stream, reportCategories, **kw):
        self.stopTestEvent = stopTestEvent
        self.stream = stream
        self.reportCategories = reportCategories
        super().__init__(**kw)


class ReportTestEvent(Event):

    """Event fired to report a test event.

    Plugins can respond to this event by producing output for the user.

    .. attribute :: testEvent

       A test event. In most cases, a
       :class:`nose2.events.TestOutcomeEvent` instance. For startTest,
       a :class:`nose2.events.StartTestEvent` instance.

    .. attribute :: stream

       The output stream. Plugins can set this to change or capture
       output.

    """

    _attrs = Event._attrs + ("testEvent", "stream")

    def __init__(self, testEvent, stream, **kw):
        self.testEvent = testEvent
        self.stream = stream
        super().__init__(**kw)


class UserInteractionEvent(Event):

    """Event fired before and after user interaction.

    Plugins that capture stdout or otherwise prevent user interaction
    should respond to this event.

    To prevent the user interaction from occurring, return ``False`` and
    set ``handled``. Otherwise, turn off whatever you are doing that
    prevents users from typing/clicking/touching/psionics/whatever.

    """

    def __init__(self, **kw):
        super().__init__(**kw)


class CommandLineArgsEvent(Event):

    """Event fired after parsing of command line arguments.

    Plugins can respond to this event by configuring themselves or other plugins
    or modifying the parsed arguments.

    .. note ::

       Many plugins register options with callbacks. By the time this event
       fires, those callbacks have already fired. So you can't use this
       event to reliably influence all plugins.

    .. attribute :: args

       Args object returned by argparse.

    """

    _attrs = Event._attrs + ("args",)

    def __init__(self, args, **kw):
        self.args = args
        super().__init__(**kw)


class CreateTestsEvent(Event):

    """Event fired before test loading.

    Plugins can take over test loading by returning a test suite and setting
    ``handled`` on this event.

    .. attribute :: loader

       Test loader instance

    .. attribute :: names

       List of test names. May be empty or ``None``.

    .. attribute :: module

       Module to load from. May be ``None``. If not ``None``, names should be
       considered relative to this module.

    """

    _attrs = Event._attrs + ("loader", "testNames", "module")

    def __init__(self, loader, testNames, module, **kw):
        self.loader = loader
        self.testNames = testNames
        self.module = module
        super().__init__(**kw)


class CreatedTestSuiteEvent(Event):
    """Event fired after test loading.

    Plugins can replace the loaded test suite by returning a test suite and
    setting ``handled`` on this event.

    .. attribute :: suite

       Test Suite instance

    """

    _attrs = Event._attrs + ("suite",)

    def __init__(self, suite, **kw):
        self.suite = suite
        super().__init__(**kw)



import logging
import re
from collections import OrderedDict

from nose2 import events, exceptions, util
from nose2.suite import LayerSuite

BRIGHT = r"\033[1m"
RESET = r"\033[0m"

__unittest = True

log = logging.getLogger(__name__)


class MissingParentLayer(Exception):
    pass


class Layers(events.Plugin):
    alwaysOn = True

    def startTestRun(self, event):
        event.suite = self.make_suite(event.suite, self.session.testLoader.suiteClass)

    def get_layers_from_suite(self, suite, suiteClass):
        top_layer = suiteClass()
        layers_dict = OrderedDict()
        for test in self.flatten_suite(suite):
            layer = getattr(test, "layer", None)
            if layer:
                if layer not in layers_dict:
                    layers_dict[layer] = LayerSuite(self.session, layer=layer)
                layers_dict[layer].addTest(test)
            else:
                top_layer.addTest(test)
        self.get_parent_layers(layers_dict)
        return top_layer, layers_dict

    def get_parent_layers(self, layers_dict):
        while True:
            missing_parents = []
            for layer in layers_dict.keys():
                for parent in layer.__bases__:
                    if parent is object:
                        continue
                    if parent not in layers_dict:
                        missing_parents.append(parent)
            if not missing_parents:
                break
            for parent in missing_parents:
                layers_dict[parent] = LayerSuite(self.session, layer=parent)

    def make_suite(self, suite, suiteClass):
        top_layer, layers_dict = self.get_layers_from_suite(suite, suiteClass)
        tree = {}
        unresolved_layers = self.update_layer_tree(tree, layers_dict.keys())
        while unresolved_layers:
            remaining = self.update_layer_tree(tree, unresolved_layers)
            if len(remaining) == len(unresolved_layers):
                raise exceptions.LoadTestsFailure(
                    "Could not resolve layer dependencies"
                )
            unresolved_layers = remaining
        for layer in tree.keys():
            if layer and layer not in layers_dict:
                layers_dict[layer] = LayerSuite(self.session, layer=layer)
        self.tree_to_suite(tree, None, top_layer, layers_dict)
        return top_layer

    @classmethod
    def update_layer_tree(cls, tree, layers):
        remaining = []
        for layer in layers:
            try:
                cls.add_layer_to_tree(tree, layer)
            except MissingParentLayer:
                remaining.append(layer)
        return remaining

    @classmethod
    def insert_mixins(cls, tree, layer, outer):
        mixins = getattr(layer, "mixins", None)
        if not mixins:
            return outer
        last = outer
        for mixin in mixins:
            mixin_ancestor = cls.get_oldest_parent(mixin)
            if last is None:
                tree.setdefault(None, []).append(mixin_ancestor)
            else:
                # The mixin_ancestor can be a layer that has been added to the
                # tree already. If so, it should a base layer, since it's the
                # last ancestor. We need to remove it from there, and insert it
                # in the "last" layer.
                if mixin_ancestor in tree[None]:
                    tree[None].remove(mixin_ancestor)
                tree[last].append(mixin_ancestor)
                if mixin_ancestor not in tree:
                    tree[mixin_ancestor] = []
            if mixin not in tree:
                tree[mixin] = []
            last = mixin
        return last

    @classmethod
    def insert_layer(cls, tree, layer, outer):
        if outer is object:
            outer = cls.insert_mixins(tree, layer, None)
        elif outer in tree:
            outer = cls.insert_mixins(tree, layer, outer)
        else:
            err = f"{outer} not found in {tree}"
            raise exceptions.LoadTestsFailure(err)
        if outer is None:
            tree.setdefault(None, []).append(layer)
        else:
            tree[outer].append(layer)
        tree[layer] = []

    @staticmethod
    def get_parents_from_tree(layer, tree):
        parents = []
        for key, value in tree.items():
            if layer in value:
                parents.append(key)

    @classmethod
    def get_oldest_parent(cls, layer):
        # FIXME: we assume that there is only one oldest parent
        # it should be the case most of the time but it will break sometimes.
        oldest = True
        for parent in layer.__bases__:
            if parent in [None, object]:
                continue
            else:
                oldest = False
                return cls.get_oldest_parent(parent)
        if oldest:
            return layer

    @classmethod
    def add_layer_to_tree(cls, tree, layer):
        parents = layer.__bases__
        if not parents:
            err = "Invalid layer {0}: should at least inherit from `object`"
            raise exceptions.LoadTestsFailure(err.format(layer))
        for parent in parents:
            if parent not in tree and parent is not object:
                raise MissingParentLayer
        # if we reached that point, then all the parents are in the tree
        # if there are multiple parents, we first try to get the closest
        # to the current layer.
        for parent in parents:
            if not cls.get_parents_from_tree(parent, tree):
                cls.insert_layer(tree, layer, parent)
                return
        raise exceptions.LoadTestsFailure(f"Failed to add {layer}")

    @classmethod
    def tree_to_suite(cls, tree, key, suite, layers):
        _suite = layers.get(key, None)
        if _suite:
            suite.addTest(_suite)
            suite = _suite
        sublayers = tree.get(key, [])
        # ensure that layers with a set order are in order
        sublayers.sort(key=cls.get_layer_position)
        for layer in sublayers:
            cls.tree_to_suite(tree, layer, suite, layers)

    @classmethod
    def flatten_suite(cls, suite):
        out = []
        for test in suite:
            try:
                out.extend(cls.flatten_suite(test))
            except TypeError:
                out.append(test)
        return out

    @staticmethod
    def get_layer_position(layer):
        pos = getattr(layer, "position", None)
        # ... lame
        if pos is not None:
            key = "%04d" % pos
        else:
            key = layer.__name__
        return key


class LayerReporter(events.Plugin):
    commandLineSwitch = (
        None,
        "layer-reporter",
        "Add layer information to test reports",
    )
    configSection = "layer-reporter"

    def __init__(self):
        self.indent = self.config.as_str("indent", "  ")
        self.colors = self.config.as_bool("colors", False)
        self.highlight_words = self.config.as_list(
            "highlight-words", ["A", "having", "should"]
        )
        self.highlight_re = re.compile(r"\b(%s)\b" % "|".join(self.highlight_words))
        self.layersReported = set()

    def reportStartTest(self, event):
        if self.session.verbosity < 2:
            return
        test = event.testEvent.test
        layer = getattr(test, "layer", None)
        if not layer:
            return
        for ix, lys in enumerate(util.ancestry(layer)):
            for layer in lys:
                if layer not in self.layersReported:
                    desc = self.describeLayer(layer)
                    event.stream.writeln(f"{self.indent * ix}{desc}")
                    self.layersReported.add(layer)
        event.stream.write(self.indent * (ix + 1))

    def describeLayer(self, layer):
        return self.format(getattr(layer, "description", layer.__name__))

    def format(self, st):
        if self.colors:
            return self.highlight_re.sub(rf"{BRIGHT}\1{RESET}", st)
        return st

    def describeTest(self, event):
        if hasattr(event.test, "methodDescription"):
            event.description = self.format(event.test.methodDescription())
        if event.errorList and hasattr(event.test, "layer"):
            # walk back layers to build full description
            self.describeLayers(event)
        # we need to remove "\n" from description to keep a well indented report when
        # tests have docstrings
        # see https://github.com/nose-devs/nose2/issues/327 for more information
        event.description = event.description.replace("\n", " ")

    def describeLayers(self, event):
        desc = [event.description]
        base = event.test.layer
        for layer in base.__mro__ + getattr(base, "mixins", ()):
            if layer is object:
                continue
            desc.append(self.describeLayer(layer))
        desc.reverse()
        event.description = " ".join(desc)


# for debugging
# def printtree(suite, indent=''):
#     import unittest
#     six.print_('%s%s ->' % (indent, getattr(suite, 'layer', 'no layer')))
#     for test in suite:
#         if isinstance(test, unittest.BaseTestSuite):
#             printtree(test, indent + '  ')
#         else:
#             six.print_('%s %s' % (indent, test))
#     six.print_('%s<- %s' % (indent, getattr(suite, 'layer', 'no layer')))



import os
import tempfile
from xml.etree import ElementTree as ET

from nose2.tests._common import FunctionalTestCase


def read_report(path):
    with open(path, encoding="utf-8") as f:
        return ET.parse(f).getroot()


class TestSubtests(FunctionalTestCase):
    def test_success(self):
        proc = self.runIn(
            "scenario/subtests", "-v", "test_subtests.Case.test_subtest_success"
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertTestRunOutputMatches(proc, stderr="test_subtest_success")
        self.assertTestRunOutputMatches(proc, stderr="OK")
        self.assertEqual(proc.poll(), 0)

    def test_failure(self):
        proc = self.runIn(
            "scenario/subtests", "-v", "test_subtests.Case.test_subtest_failure"
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_failure.*\(i=1\)")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_failure.*\(i=3\)")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_failure.*\(i=5\)")
        self.assertTestRunOutputMatches(proc, stderr=r"FAILED \(failures=3\)")
        self.assertEqual(proc.poll(), 1)

    def test_error(self):
        proc = self.runIn(
            "scenario/subtests", "-v", "test_subtests.Case.test_subtest_error"
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_error.*\(i=0\)")
        self.assertTestRunOutputMatches(proc, stderr="RuntimeError: 0")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_error.*\(i=1\)")
        self.assertTestRunOutputMatches(proc, stderr="RuntimeError: 1")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_error.*\(i=2\)")
        self.assertTestRunOutputMatches(proc, stderr="RuntimeError: 2")
        self.assertTestRunOutputMatches(proc, stderr=r"FAILED \(errors=3\)")
        self.assertEqual(proc.poll(), 1)

    def test_expected_failure(self):
        proc = self.runIn(
            "scenario/subtests",
            "-v",
            "test_subtests.Case.test_subtest_expected_failure",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertTestRunOutputMatches(
            proc, stderr="test_subtest_expected_failure.*expected failure"
        )
        self.assertTestRunOutputMatches(proc, stderr=r"OK \(expected failures=1\)")
        self.assertEqual(proc.poll(), 0)

    def test_message(self):
        proc = self.runIn(
            "scenario/subtests", "-v", "test_subtests.Case.test_subtest_message"
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertTestRunOutputMatches(
            proc, stderr=r"test_subtest_message.*\[msg\] \(i=1\)"
        )
        self.assertTestRunOutputMatches(
            proc, stderr=r"test_subtest_message.*\[msg\] \(i=3\)"
        )
        self.assertTestRunOutputMatches(
            proc, stderr=r"test_subtest_message.*\[msg\] \(i=5\)"
        )
        self.assertTestRunOutputMatches(proc, stderr=r"FAILED \(failures=3\)")
        self.assertEqual(proc.poll(), 1)

    def test_all(self):
        proc = self.runIn("scenario/subtests", "-v", "test_subtests")
        self.assertTestRunOutputMatches(proc, stderr="Ran 5 tests")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_failure.*\(i=1\)")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_failure.*\(i=3\)")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_failure.*\(i=5\)")
        self.assertTestRunOutputMatches(
            proc, stderr=r"test_subtest_message.*\[msg\] \(i=1\)"
        )
        self.assertTestRunOutputMatches(
            proc, stderr=r"test_subtest_message.*\[msg\] \(i=3\)"
        )
        self.assertTestRunOutputMatches(
            proc, stderr=r"test_subtest_message.*\[msg\] \(i=5\)"
        )
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_error.*\(i=0\)")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_error.*\(i=1\)")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_error.*\(i=2\)")
        self.assertTestRunOutputMatches(
            proc, stderr=r"FAILED \(failures=6, errors=3, expected failures=1\)"
        )
        self.assertEqual(proc.poll(), 1)


class TestSubtestsMultiprocess(FunctionalTestCase):
    def test_success(self):
        proc = self.runIn(
            "scenario/subtests",
            "--plugin=nose2.plugins.mp",
            "--processes=2",
            "-v",
            "test_subtests.Case.test_subtest_success",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertTestRunOutputMatches(proc, stderr="test_subtest_success")
        self.assertTestRunOutputMatches(proc, stderr="OK")
        self.assertEqual(proc.poll(), 0)

    def test_failure(self):
        proc = self.runIn(
            "scenario/subtests",
            "--plugin=nose2.plugins.mp",
            "--processes=2",
            "-v",
            "test_subtests.Case.test_subtest_failure",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_failure.*\(i=1\)")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_failure.*\(i=3\)")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_failure.*\(i=5\)")
        self.assertTestRunOutputMatches(proc, stderr=r"FAILED \(failures=3\)")
        self.assertEqual(proc.poll(), 1)

    def test_error(self):
        proc = self.runIn(
            "scenario/subtests",
            "--plugin=nose2.plugins.mp",
            "--processes=2",
            "-v",
            "test_subtests.Case.test_subtest_error",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_error.*\(i=0\)")
        self.assertTestRunOutputMatches(proc, stderr="RuntimeError: 0")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_error.*\(i=1\)")
        self.assertTestRunOutputMatches(proc, stderr="RuntimeError: 1")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_error.*\(i=2\)")
        self.assertTestRunOutputMatches(proc, stderr="RuntimeError: 2")
        self.assertTestRunOutputMatches(proc, stderr=r"FAILED \(errors=3\)")
        self.assertEqual(proc.poll(), 1)

    def test_expected_failure(self):
        proc = self.runIn(
            "scenario/subtests",
            "--plugin=nose2.plugins.mp",
            "--processes=2",
            "-v",
            "test_subtests.Case.test_subtest_expected_failure",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertTestRunOutputMatches(
            proc, stderr="test_subtest_expected_failure.*expected failure"
        )
        self.assertTestRunOutputMatches(proc, stderr=r"OK \(expected failures=1\)")
        self.assertEqual(proc.poll(), 0)

    def test_message(self):
        proc = self.runIn(
            "scenario/subtests",
            "--plugin=nose2.plugins.mp",
            "--processes=2",
            "-v",
            "test_subtests.Case.test_subtest_message",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertTestRunOutputMatches(
            proc, stderr=r"test_subtest_message.*\[msg\] \(i=1\)"
        )
        self.assertTestRunOutputMatches(
            proc, stderr=r"test_subtest_message.*\[msg\] \(i=3\)"
        )
        self.assertTestRunOutputMatches(
            proc, stderr=r"test_subtest_message.*\[msg\] \(i=5\)"
        )
        self.assertTestRunOutputMatches(proc, stderr=r"FAILED \(failures=3\)")
        self.assertEqual(proc.poll(), 1)

    def test_all(self):
        proc = self.runIn(
            "scenario/subtests",
            "--plugin=nose2.plugins.mp",
            "--processes=2",
            "-v",
            "test_subtests",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 5 tests")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_failure.*\(i=1\)")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_failure.*\(i=3\)")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_failure.*\(i=5\)")
        self.assertTestRunOutputMatches(
            proc, stderr=r"test_subtest_message.*\[msg\] \(i=1\)"
        )
        self.assertTestRunOutputMatches(
            proc, stderr=r"test_subtest_message.*\[msg\] \(i=3\)"
        )
        self.assertTestRunOutputMatches(
            proc, stderr=r"test_subtest_message.*\[msg\] \(i=5\)"
        )
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_error.*\(i=0\)")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_error.*\(i=1\)")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_error.*\(i=2\)")
        self.assertTestRunOutputMatches(
            proc, stderr=r"FAILED \(failures=6, errors=3, expected failures=1\)"
        )
        self.assertEqual(proc.poll(), 1)


class TestSubtestsJunitXml(FunctionalTestCase):
    def setUp(self):
        super().setUp()
        tmp = tempfile.NamedTemporaryFile(delete=False)
        tmp.close()
        self.junit_report = tmp.name

    def tearDown(self):
        os.remove(self.junit_report)

    def test_success(self):
        proc = self.runIn(
            "scenario/subtests",
            "--plugin=nose2.plugins.junitxml",
            "--junit-xml",
            f"--junit-xml-path={self.junit_report}",
            "-v",
            "test_subtests.Case.test_subtest_success",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertTestRunOutputMatches(proc, stderr="OK")
        self.assertEqual(proc.poll(), 0)

        tree = read_report(self.junit_report)
        self.assertEqual(tree.get("tests"), "1")
        self.assertEqual(tree.get("failures"), "0")
        self.assertEqual(tree.get("errors"), "0")
        self.assertEqual(tree.get("skipped"), "0")
        self.assertEqual(len(tree.findall("testcase")), 1)

        for test_case in tree.findall("testcase"):
            self.assertEqual(test_case.get("name"), "test_subtest_success")

    def test_failure(self):
        proc = self.runIn(
            "scenario/subtests",
            "--plugin=nose2.plugins.junitxml",
            "--junit-xml",
            f"--junit-xml-path={self.junit_report}",
            "-v",
            "test_subtests.Case.test_subtest_failure",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertTestRunOutputMatches(proc, stderr="FAILED")
        self.assertEqual(proc.poll(), 1)

        tree = read_report(self.junit_report)
        self.assertEqual(tree.get("tests"), "1")
        self.assertEqual(tree.get("failures"), "3")
        self.assertEqual(tree.get("errors"), "0")
        self.assertEqual(tree.get("skipped"), "0")
        self.assertEqual(len(tree.findall("testcase")), 3)

        for index, test_case in enumerate(tree.findall("testcase")):
            self.assertEqual(
                test_case.get("name"),
                f"test_subtest_failure (i={index * 2 + 1})",
            )

    def test_error(self):
        proc = self.runIn(
            "scenario/subtests",
            "--plugin=nose2.plugins.junitxml",
            "--junit-xml",
            f"--junit-xml-path={self.junit_report}",
            "-v",
            "test_subtests.Case.test_subtest_error",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertTestRunOutputMatches(proc, stderr="FAILED")
        self.assertEqual(proc.poll(), 1)

        tree = read_report(self.junit_report)
        self.assertEqual(tree.get("tests"), "1")
        self.assertEqual(tree.get("failures"), "0")
        self.assertEqual(tree.get("errors"), "3")
        self.assertEqual(tree.get("skipped"), "0")
        self.assertEqual(len(tree.findall("testcase")), 3)

        for index, test_case in enumerate(tree.findall("testcase")):
            self.assertEqual(test_case.get("name"), f"test_subtest_error (i={index})")

    def test_expected_failure(self):
        proc = self.runIn(
            "scenario/subtests",
            "--plugin=nose2.plugins.junitxml",
            "--junit-xml",
            f"--junit-xml-path={self.junit_report}",
            "-v",
            "test_subtests.Case.test_subtest_expected_failure",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertTestRunOutputMatches(proc, stderr="OK")
        self.assertEqual(proc.poll(), 0)

        tree = read_report(self.junit_report)
        self.assertEqual(tree.get("tests"), "1")
        self.assertEqual(tree.get("failures"), "0")
        self.assertEqual(tree.get("errors"), "0")
        self.assertEqual(tree.get("skipped"), "1")
        self.assertEqual(len(tree.findall("testcase")), 1)

        for test_case in tree.findall("testcase"):
            self.assertEqual(test_case.get("name"), "test_subtest_expected_failure")

    def test_message(self):
        proc = self.runIn(
            "scenario/subtests",
            "--plugin=nose2.plugins.junitxml",
            "--junit-xml",
            f"--junit-xml-path={self.junit_report}",
            "-v",
            "test_subtests.Case.test_subtest_message",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertTestRunOutputMatches(proc, stderr="FAILED")
        self.assertEqual(proc.poll(), 1)

        tree = read_report(self.junit_report)
        self.assertEqual(tree.get("tests"), "1")
        self.assertEqual(tree.get("failures"), "3")
        self.assertEqual(tree.get("errors"), "0")
        self.assertEqual(tree.get("skipped"), "0")
        self.assertEqual(len(tree.findall("testcase")), 3)

        for index, test_case in enumerate(tree.findall("testcase")):
            self.assertEqual(
                test_case.get("name"),
                f"test_subtest_message [msg] (i={index * 2 + 1})",
            )

    def test_all(self):
        proc = self.runIn(
            "scenario/subtests",
            "--plugin=nose2.plugins.junitxml",
            "--junit-xml",
            f"--junit-xml-path={self.junit_report}",
            "-v",
            "test_subtests",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 5 tests")
        self.assertTestRunOutputMatches(proc, stderr="FAILED")
        self.assertEqual(proc.poll(), 1)

        tree = read_report(self.junit_report)
        self.assertEqual(tree.get("tests"), "5")
        self.assertEqual(tree.get("failures"), "6")
        self.assertEqual(tree.get("errors"), "3")
        self.assertEqual(tree.get("skipped"), "1")
        self.assertEqual(len(tree.findall("testcase")), 11)


class TestSubtestsFailFast(FunctionalTestCase):
    def test_failure(self):
        proc = self.runIn(
            "scenario/subtests", "-v", "test_subtests.Case.test_subtest_failure"
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_failure.*\(i=1\)")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_failure.*\(i=3\)")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_failure.*\(i=5\)")
        self.assertTestRunOutputMatches(proc, stderr=r"FAILED \(failures=3\)")
        self.assertEqual(proc.poll(), 1)

    def test_failfast(self):
        proc = self.runIn(
            "scenario/subtests",
            "--fail-fast",
            "-v",
            "test_subtests.Case.test_subtest_failure",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertTestRunOutputMatches(proc, stderr=r"test_subtest_failure.*\(i=1\)")
        self.assertTestRunOutputMatches(proc, stderr=r"FAILED \(failures=1\)")
        self.assertEqual(proc.poll(), 1)



from nose2.tests._common import FunctionalTestCase


class DecoratorsTests(FunctionalTestCase):
    def test_with_setup(self):
        process = self.runIn("scenario/decorators", "test_decorators.test_with_setup")

        self.assertTestRunOutputMatches(process, stderr="Ran 1 test")
        self.assertEqual(process.poll(), 0, process.stderr.getvalue())

    def test_with_teardown(self):
        process = self.runIn(
            "scenario/decorators",
            "test_decorators.test_with_teardown",
            "test_decorators.test_teardown_ran",
        )

        self.assertTestRunOutputMatches(process, stderr="Ran 2 test")
        self.assertEqual(process.poll(), 0, process.stderr.getvalue())



import logging
import unittest

from .common import NormalTest, NormalTestTwo, UniqueResource  # noqa: F401

log = logging.getLogger(__name__)


class Layer1:
    @classmethod
    def setUp(cls):
        log.info("Called setup in layer 1")
        cls.unique_resource = UniqueResource()
        cls.unique_resource.lock()

    @classmethod
    def tearDown(cls):
        log.info("Called teardown in layer 2")
        cls.unique_resource.unlock()


class Layer2:
    @classmethod
    def setUp(cls):
        log.info("Called setup in layer 2")
        cls.unique_resource = UniqueResource()
        cls.unique_resource.lock()

    @classmethod
    def tearDown(cls):
        log.info("Called teardown in layer 2")
        cls.unique_resource.unlock()


class Layer3(Layer2):
    @classmethod
    def setUp(cls):
        log.info("Called setup in layer 3")

    @classmethod
    def tearDown(cls):
        log.info("Called teardown in layer 3")


class LayerTest1(unittest.TestCase):
    layer = Layer1

    def test(self):
        self.assertTrue(self.layer.unique_resource.used)


class LayerTest2(unittest.TestCase):
    layer = Layer2

    def test(self):
        self.assertTrue(self.layer.unique_resource.used)


class LayerTest3(unittest.TestCase):
    layer = Layer2

    def test(self):
        self.assertTrue(self.layer.unique_resource.used)



from nose2 import events, session
from nose2.tests._common import TestCase


class Example(events.Plugin):
    commandLineSwitch = ("X", "xxx", "triple x")

    def testOutcome(self, event):
        pass


class TestPluginApi(TestCase):
    def setUp(self):
        self.session = session.Session()
        self.plug = Example(session=self.session)
        super(TestCase, self).setUp()

    def test_add_option_adds_option(self):
        helpt = self.session.argparse.format_help()
        assert "-X, --xxx" in helpt, (
            "commandLineSwitch arg not found in help text: %s" % helpt
        )

    def test_short_opt_registers_plugin(self):
        args, argv = self.session.argparse.parse_known_args(["-X"])
        assert self.plug in self.session.plugins
        assert (
            self.plug in self.session.hooks.testOutcome.plugins
        ), "short opt did not register plugin"

    def test_long_opt_registers_plugin(self):
        args, argv = self.session.argparse.parse_known_args(["--xxx"])
        assert self.plug in self.session.plugins
        assert (
            self.plug in self.session.hooks.testOutcome.plugins
        ), "long opt did not register plugin"



from nose2.tests._common import FunctionalTestCase


class TestLoadTestsPlugin(FunctionalTestCase):
    def test_simple(self):
        proc = self.runIn(
            "scenario/load_tests", "-v", "--plugin=nose2.plugins.loader.loadtests"
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 6 tests")
        self.assertTestRunOutputMatches(proc, stderr="test_a..test_simple")
        self.assertTestRunOutputMatches(proc, stderr="test_b..test_simple")
        self.assertTestRunOutputMatches(proc, stderr="test_c..test_simple")
        self.assertTestRunOutputMatches(proc, stderr="test_d..test_simple")
        self.assertTestRunOutputMatches(proc, stderr="test_a..test_filter")
        self.assertTestRunOutputMatches(proc, stderr="test_c..test_filter")
        self.assertEqual(proc.poll(), 0)

    def test_package(self):
        proc = self.runIn(
            "scenario/load_tests_pkg",
            "-v",
            "-c=" "nose2/tests/functional/support/scenario/load_tests_pkg/unittest.cfg",
            "--plugin=nose2.plugins.loader.loadtests",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 2 tests")
        self.assertTestRunOutputMatches(
            proc, stderr="test..ltpkg.tests.test_find_these.Test"
        )
        self.assertTestRunOutputMatches(
            proc, stderr="test..ltpkg2.tests.load_tests.<locals>.Test"
        )

    def test_project_directory_inside_package(self):
        proc = self.runIn(
            "scenario/load_tests_pkg/ltpkg/tests",
            "-v",
            "-c=" "nose2/tests/functional/support/scenario/load_tests_pkg/unittest.cfg",
            "--plugin=nose2.plugins.loader.loadtests",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertTestRunOutputMatches(
            proc, stderr="test..ltpkg.tests.test_find_these.Test"
        )



import datetime
import logging
import os
import sys
import time
import unittest
from xml.etree import ElementTree as ET

from nose2 import events, loader, result, session, tools
from nose2.plugins import junitxml, logcapture
from nose2.plugins.loader import functions, generators, parameters, testcases
from nose2.tests._common import TestCase


def _fromisoformat(date_str):
    # use fromisoformat when it is available, but failover to strptime for python2 and
    # older python3 versions
    if hasattr(datetime.datetime, "fromisoformat"):
        return datetime.datetime.fromisoformat(date_str)
    return datetime.datetime.strptime(date_str, "%Y-%m-%dT%H:%M:%S.%f")


class TestJunitXmlPlugin(TestCase):
    _RUN_IN_TEMP = True

    BAD_FOR_XML_U = "A\x07 B\x0B C\x10 D\uD900 " "E\uFFFE F\x80 G\x90 H\uFDDD"
    # UTF-8 string with double null (invalid)
    BAD_FOR_XML_B = (
        b"A\x07 B\x0b C\x10 D\xed\xa4\x80 "
        b"E\xef\xbf\xbe F\xc2\x80 G\xc2\x90 H\xef\xb7\x9d "
        b"\x00\x00"
    )

    # "byte" strings in PY2 and unicode in py3 works as expected will
    # will translate surrogates into UTF-16 characters  so BAD_FOR_XML_U
    # should have 8 letters follows by 0xFFFD, but only 4 when keeping
    # the discouraged/restricted ranges. Respectively:
    # "A\uFFFD B\uFFFD C\uFFFD D\uFFFD E\uFFFD F\uFFFD G\uFFFD H\uFFFD"
    # "A\uFFFD B\uFFFD C\uFFFD D\uFFFD E\uFFFD F\x80 G\x90 H\uFDDD"
    #
    # In Python 2 Invalid ascii characters seem to get escaped out as part
    # of tracebace.format_traceback so full and partial replacements are:
    # "A\uFFFD B\uFFFD C\uFFFD D\\\\ud900 E\\\\ufffe F\\\\x80 G\\\\x90 H\\\\ufddd"
    # "A\uFFFD B\uFFFD C\uFFFD D\\\\ud900 E\\\\ufffe F\\\\x80 G\\\\x90 H\\\\ufddd"
    #
    # Byte strings in py3 as errors are replaced by their representation string
    # So these will be safe and not have any replacements
    # "b'A\\x07 B\\x0b C\\x10 D\\xed\\xa4\\x80 E\\xef\\xbf\\xbe F\\xc2\\x80
    # G\\xc2\\x90 H\\xef\\xb7\\x9d \\x00\\x00"

    if sys.maxunicode <= 0xFFFF:
        EXPECTED_RE = "^[\x09\x0A\x0D\x20\x21-\uD7FF\uE000-\uFFFD]*$"
        EXPECTED_RE_SAFE = (
            "^[\x09\x0A\x0D\x20\x21-\x7E\x85" "\xA0-\uD7FF\uE000-\uFDCF\uFDF0-\uFFFD]*$"
        )
    else:
        EXPECTED_RE = "^[\x09\x0A\x0D\x20\x21-\uD7FF\uE000-\uFFFD" "\u10000-\u10FFFF]*$"
        EXPECTED_RE_SAFE = (
            "^[\x09\x0A\x0D\x20\x21-\x7E\x85"
            "\xA0-\uD7FF\uE000-\uFDCF\uFDF0-\uFFFD"
            "\u10000-\u1FFFD\u20000-\u2FFFD"
            "\u30000-\u3FFFD\u40000-\u4FFFD"
            "\u50000-\u5FFFD\u60000-\u6FFFD"
            "\u70000-\u7FFFD\u80000-\u8FFFD"
            "\u90000-\u8FFFD\uA0000-\uAFFFD"
            "\uB0000-\uBFFFD\uC0000-\uCFFFD"
            "\uD0000-\uDFFFD\uE0000-\uEFFFD"
            "\uF0000-\uFFFFD\u100000-\u10FFFD]*$"
        )

    def setUp(self):
        super().setUp()
        self.session = session.Session()
        self.loader = loader.PluggableTestLoader(self.session)
        self.result = result.PluggableTestResult(self.session)
        self.plugin = junitxml.JUnitXmlReporter(session=self.session)
        self.plugin.register()

        class Test(unittest.TestCase):
            def test(self):
                pass

            def test_chdir(self):
                TEMP_SUBFOLDER = "test_chdir"

                os.mkdir(TEMP_SUBFOLDER)
                os.chdir(TEMP_SUBFOLDER)

            def test_fail(self):
                assert False

            def test_err(self):
                1 / 0  # noqa: B018 useless expression

            def test_skip(self):
                self.skipTest("skip")

            def test_skip_no_reason(self):
                self.skipTest("")

            def test_bad_xml(self):
                raise RuntimeError(TestJunitXmlPlugin.BAD_FOR_XML_U)

            def test_bad_xml_b(self):
                raise RuntimeError(TestJunitXmlPlugin.BAD_FOR_XML_B)

            def test_gen(self):
                def check(a, b):
                    # workaround:
                    # for the test which ensures that the timestamp increases between
                    # generator test runs, insert a very small sleep
                    #
                    # on py3.10 on Windows, we have observed both of these tests getting
                    # the same timestamp
                    time.sleep(0.001)
                    self.assertEqual(a, b)

                yield check, 1, 1
                yield check, 1, 2

            @tools.params(1, 2, 3)
            def test_params(self, p):
                self.assertEqual(p, 2)

            def test_with_log(self):
                logging.info("log message")

        self.case = Test

    def test_success_added_to_xml(self):
        test = self.case("test")
        test(self.result)
        self.assertEqual(self.plugin.numtests, 1)
        self.assertEqual(len(self.plugin.tree.findall("testcase")), 1)

    def test_failure_includes_traceback(self):
        test = self.case("test_fail")
        test(self.result)
        case = self.plugin.tree.find("testcase")
        failure = case.find("failure")
        assert failure is not None
        assert "Traceback" in failure.text

    def test_error_bad_xml(self):
        self.plugin.keep_restricted = False
        test = self.case("test_bad_xml")
        test(self.result)
        case = self.plugin.tree.find("testcase")
        error = case.find("error")
        self.assertRegex(error.text, self.EXPECTED_RE_SAFE)

    def test_error_bad_xml_keep(self):
        self.plugin.keep_restricted = True
        test = self.case("test_bad_xml")
        test(self.result)
        case = self.plugin.tree.find("testcase")
        error = case.find("error")
        self.assertRegex(error.text, self.EXPECTED_RE)

    def test_error_bad_xml_b(self):
        self.plugin.keep_restricted = False
        test = self.case("test_bad_xml_b")
        test(self.result)
        case = self.plugin.tree.find("testcase")
        error = case.find("error")
        assert error is not None
        self.assertRegex(error.text, self.EXPECTED_RE_SAFE)

    def test_error_bad_xml_b_keep(self):
        self.plugin.keep_restricted = True
        test = self.case("test_bad_xml_b")
        test(self.result)
        case = self.plugin.tree.find("testcase")
        error = case.find("error")
        assert error is not None
        self.assertRegex(error.text, self.EXPECTED_RE)

    def test_error_includes_traceback(self):
        test = self.case("test_err")
        test(self.result)
        case = self.plugin.tree.find("testcase")
        error = case.find("error")
        assert error is not None
        assert "Traceback" in error.text

    def test_skip_includes_skipped(self):
        test = self.case("test_skip")
        test(self.result)
        case = self.plugin.tree.find("testcase")
        skip = case.find("skipped")
        assert skip is not None
        self.assertEqual(skip.get("message"), "test skipped: skip")
        self.assertEqual(skip.text, "skip")

    def test_skip_includes_skipped_no_reason(self):
        test = self.case("test_skip_no_reason")
        test(self.result)
        case = self.plugin.tree.find("testcase")
        skip = case.find("skipped")
        assert skip is not None
        self.assertIsNone(skip.get("message"))
        self.assertIsNone(skip.text)

    def test_generator_timestamp_increases(self):
        gen = generators.Generators(session=self.session)
        gen.register()
        event = events.LoadFromTestCaseEvent(self.loader, self.case)
        self.session.hooks.loadTestsFromTestCase(event)
        cases = event.extraTests
        for case in cases:
            case(self.result)
        xml = self.plugin.tree.findall("testcase")
        self.assertEqual(len(xml), 2)
        test1_timestamp_str = xml[0].get("timestamp")
        test1_timestamp = _fromisoformat(test1_timestamp_str)
        test2_timestamp_str = xml[1].get("timestamp")
        test2_timestamp = _fromisoformat(test2_timestamp_str)
        self.assertGreater(test2_timestamp, test1_timestamp)

    def test_generator_test_name_correct(self):
        gen = generators.Generators(session=self.session)
        gen.register()
        event = events.LoadFromTestCaseEvent(self.loader, self.case)
        self.session.hooks.loadTestsFromTestCase(event)
        cases = event.extraTests
        for case in cases:
            case(self.result)
        xml = self.plugin.tree.findall("testcase")
        self.assertEqual(len(xml), 2)
        self.assertEqual(xml[0].get("name"), "test_gen:1")
        self.assertEqual(xml[1].get("name"), "test_gen:2")

    def test_generator_test_full_name_correct(self):
        gen = generators.Generators(session=self.session)
        gen.register()
        self.plugin.test_fullname = True
        event = events.LoadFromTestCaseEvent(self.loader, self.case)
        self.session.hooks.loadTestsFromTestCase(event)
        cases = event.extraTests
        for case in cases:
            case(self.result)
        xml = self.plugin.tree.findall("testcase")
        self.assertEqual(len(xml), 2)
        self.assertEqual(xml[0].get("name"), "test_gen:1 (1, 1)")
        self.assertEqual(xml[1].get("name"), "test_gen:2 (1, 2)")

    def test_function_classname_is_module(self):
        fun = functions.Functions(session=self.session)
        fun.register()

        def test_func():
            pass

        cases = fun._createTests(test_func)
        self.assertEqual(len(cases), 1)
        cases[0](self.result)
        xml = self.plugin.tree.findall("testcase")
        self.assertEqual(len(xml), 1)
        self.assertTrue(xml[0].get("classname").endswith("test_junitxml"))

    def test_params_test_name_correct(self):
        # param test loading is a bit more complex than generator
        # loading. XXX -- can these be reconciled so they both
        # support exclude and also both support loadTestsFromTestCase?
        plug1 = parameters.Parameters(session=self.session)
        plug1.register()
        plug2 = testcases.TestCaseLoader(session=self.session)
        plug2.register()
        # need module to fire top-level event

        class Mod:
            pass

        m = Mod()
        m.Test = self.case
        event = events.LoadFromModuleEvent(self.loader, m)
        self.session.hooks.loadTestsFromModule(event)
        for case in event.extraTests:
            case(self.result)
        xml = self.plugin.tree.findall("testcase")
        self.assertEqual(len(xml), 13)
        params = [x for x in xml if x.get("name").startswith("test_params")]
        self.assertEqual(len(params), 3)
        self.assertEqual(params[0].get("name"), "test_params:1")
        self.assertEqual(params[1].get("name"), "test_params:2")
        self.assertEqual(params[2].get("name"), "test_params:3")

    def test_params_test_full_name_correct(self):
        plug1 = parameters.Parameters(session=self.session)
        plug1.register()
        plug2 = testcases.TestCaseLoader(session=self.session)
        plug2.register()
        # need module to fire top-level event

        class Mod:
            pass

        m = Mod()
        m.Test = self.case
        event = events.LoadFromModuleEvent(self.loader, m)
        self.plugin.test_fullname = True
        self.session.hooks.loadTestsFromModule(event)
        for case in event.extraTests:
            case(self.result)
        xml = self.plugin.tree.findall("testcase")
        self.assertEqual(len(xml), 13)
        params = [x for x in xml if x.get("name").startswith("test_params")]
        self.assertEqual(len(params), 3)
        self.assertEqual(params[0].get("name"), "test_params:1 (1)")
        self.assertEqual(params[1].get("name"), "test_params:2 (2)")
        self.assertEqual(params[2].get("name"), "test_params:3 (3)")

    def test_writes_xml_file_at_end(self):
        test = self.case("test")
        test(self.result)
        event = events.StopTestRunEvent(None, self.result, 1, 1)
        self.plugin.stopTestRun(event)
        with open(self.plugin.path) as fh:
            tree = ET.parse(fh).getroot()
        self.assertEqual(len(tree.findall("testcase")), 1)
        case = tree.find("testcase")
        assert "time" in case.attrib
        assert "timestamp" in case.attrib
        assert "classname" in case.attrib
        self.assertEqual(case.get("name"), "test")
        self.assertEqual(tree.get("errors"), "0")
        self.assertEqual(tree.get("failures"), "0")
        self.assertEqual(tree.get("skipped"), "0")
        self.assertEqual(tree.get("tests"), "1")
        assert "time" in tree.attrib

    def test_xml_file_path_is_not_affected_by_chdir_in_test(self):
        initial_dir = os.path.realpath(os.getcwd())
        test = self.case("test_chdir")
        test(self.result)
        plugin_dir = os.path.realpath(os.path.dirname(self.plugin.path))
        assert initial_dir == plugin_dir, self.plugin.path

    def test_xml_contains_empty_system_out_without_logcapture(self):
        test = self.case("test_with_log")
        test(self.result)
        case = self.plugin.tree.find("testcase")
        system_out = case.find("system-out")
        assert system_out is not None
        assert not system_out.text

    def test_xml_contains_log_message_in_system_out_with_logcapture(self):
        self.logcapture_plugin = logcapture.LogCapture(session=self.session)
        self.logcapture_plugin.register()

        test = self.case("test_with_log")
        test(self.result)
        case = self.plugin.tree.find("testcase")
        system_out = case.find("system-out")
        assert system_out is not None
        assert "log message" in system_out.text
        assert "INFO" in system_out.text



import logging
import unittest

log = logging.getLogger(__name__)


class UniqueResource:
    _instance = None
    used = False

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super().__new__(cls, *args, **kwargs)
        return cls._instance

    def lock(self):
        if not self.used:
            self.used = True
        else:
            raise Exception("Resource already used")

    def unlock(self):
        if self.used:
            self.used = False
        else:
            raise Exception("Resource already unlocked")


class NormalTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        log.info("Called setUpClass in NormalTest")
        cls.unique_resource = UniqueResource()
        cls.unique_resource.lock()

    @classmethod
    def tearDownClass(cls):
        log.info("Called tearDownClass in NormalTest")
        cls.unique_resource.unlock()

    def test(self):
        self.assertTrue(self.unique_resource.used)


class NormalTestTwo(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        log.info("Called setUpClass in NormalTestTwo")
        cls.unique_resource = UniqueResource()
        cls.unique_resource.lock()

    @classmethod
    def tearDownClass(cls):
        log.info("Called tearDownClass in NormalTestTwo")
        cls.unique_resource.unlock()

    def test(self):
        self.assertTrue(self.unique_resource.used)



from nose2 import events, loader, session
from nose2.plugins.loader.testclasses import TestClassLoader
from nose2.tests._common import TestCase


class TestTestClassLoader(TestCase):
    def setUp(self):
        self.session = session.Session()
        self.loader = loader.PluggableTestLoader(session=self.session)
        self.plugin = TestClassLoader(session=self.session)

        class Mod:
            pass

        self.module = Mod()

        class TestA:
            def test(self):
                pass

        class TestB:
            def runTest(self):
                pass

        class TestC:
            def foo(self):
                pass

        class Test(TestCase):
            def test(self):
                pass

        self.module.TestA = TestA
        self.module.TestB = TestB
        self.module.TestC = TestC
        self.module.Test = Test

    def test_can_find_testclasses_in_module(self):
        event = events.LoadFromModuleEvent(self.loader, self.module)
        result = self.session.hooks.loadTestsFromModule(event)
        self.assertEqual(result, None)
        self.assertEqual(len(event.extraTests), 3)
        self.assertEqual(len(event.extraTests[0]._tests), 1)  # TestA
        self.assertEqual(len(event.extraTests[1]._tests), 0)  # TestB
        self.assertEqual(len(event.extraTests[2]._tests), 0)  # TestC

    def test_get_testmethod_names_can_override_name_selection(self):
        class FooIsOnlyTest(events.Plugin):
            def getTestMethodNames(self, event):
                event.handled = True
                return ["foo"] if "foo" in dir(event.testCase) else []

        foo = FooIsOnlyTest(session=self.session)
        foo.register()
        event = events.LoadFromModuleEvent(self.loader, self.module)
        result = self.session.hooks.loadTestsFromModule(event)
        self.assertEqual(result, None)
        self.assertEqual(len(event.extraTests), 3)
        self.assertEqual(len(event.extraTests[0]._tests), 0)  # TestA
        self.assertEqual(len(event.extraTests[1]._tests), 0)  # TestB
        self.assertEqual(len(event.extraTests[2]._tests), 1)  # TestC

    def test_plugins_can_exclude_test_names(self):
        class Excluder(events.Plugin):
            def getTestMethodNames(self, event):
                event.excludedNames.append("test")

        excl = Excluder(session=self.session)
        excl.register()
        event = events.LoadFromModuleEvent(self.loader, self.module)
        result = self.session.hooks.loadTestsFromModule(event)
        self.assertEqual(result, None)
        self.assertEqual(len(event.extraTests), 3)
        self.assertEqual(len(event.extraTests[0]._tests), 0)  # TestA
        self.assertEqual(len(event.extraTests[1]._tests), 0)  # TestB
        self.assertEqual(len(event.extraTests[2]._tests), 0)  # TestC


class TestFailingTestClassLoader(TestCase):
    def setUp(self):
        self.session = session.Session()
        self.loader = loader.PluggableTestLoader(session=self.session)
        self.plugin = TestClassLoader(session=self.session)

        class Mod:
            pass

        self.module = Mod()

        class TestA:
            def __init__(self):
                raise RuntimeError("Something bad happened!")

            def test(self):
                pass

        self.module.TestA = TestA

    def test_can_find_testclasses_in_module(self):
        event = events.LoadFromModuleEvent(self.loader, self.module)
        result = self.session.hooks.loadTestsFromModule(event)
        self.assertEqual(result, None)
        self.assertEqual(len(event.extraTests), 1)
        self.assertEqual(len(event.extraTests[0]._tests), 1)  # TestA
        self.assertEqual(
            event.extraTests[0]._tests[0].__class__.__name__, "LoadTestsFailure"
        )
